using DataContracts;
using System;
using System.IO;
using System.Text;
using System.Xml;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text.RegularExpressions;

namespace Ato.EN.IntegrationServices.CodeGenerationDDCTNS
{

    public partial class DDCTNS2019Validator
    {
        /// <summary>
        /// The max parameter name length is restricted by the SBR1 and SBR2 schemas - the lowest common denominator of SBR 1 is used as the default
        /// </summary>
        private int _maxParameterNameLength;

        /// <summary>
        /// The max parameter value length is restricted by the SBR1 and SBR2 schemas
        /// </summary>
        private int _maxParameterValueLength;

        /// <summary>
        /// The SBR1 and ebms3 schemas do not allow parameter names or values to be the empty string
        /// </summary>
        private string _emptyParameterValue;

        /// <summary>
        /// Initializes a new instance of the <see cref="DDCTNS2019Validator" /> class.
        /// </summary>
        /// <param name="maxParameterNameLength">Maximum length of the parameter name.</param>
        /// <param name="maxParameterValueLength">Maximum length of the parameter value.</param>
        /// <param name="emptyParameterValue">This value will be used in place of any parameter values that result in a null or empty value.</param>
        public DDCTNS2019Validator(int maxParameterNameLength = 20, int maxParameterValueLength = 4096, string emptyParameterValue = "EMPTY")
        {
            _maxParameterNameLength = maxParameterNameLength;
            _maxParameterValueLength = maxParameterValueLength;
            _emptyParameterValue = emptyParameterValue;
        }

        #region Functions
        private static IEnumerable<string> Union(IEnumerable<string> list1, IEnumerable<string> list2)
        {
            IEnumerable<string> response;

            if (list1 == null && list2 == null)
            {
                response = null;
            }
            else if (list1 == null)
            {
                response = list2.Distinct();
            }
            else if (list2 == null)
            {
                response = list1.Distinct();
            }
            else
            {
                response = list1.Union(list2);
            }

            return response;
        }

        private static bool IsMatch(int? field, string expression, RegexOptions options = RegexOptions.None)
        {
            if (field == null)
                return false;
            else
                return Regex.IsMatch(Convert.ToString(field.Value), expression, options);
        }

        private static bool IsMatch(string field, string expression, RegexOptions options = RegexOptions.None)
        {
            if (field == null)
                return false;
            else
                return Regex.IsMatch(field, expression, options);
        }

        // This is just context and tuple counts where they are integer values - easier that changing the parsing logic to just return the value
        private static int Count(int count)
        {
            return count;
        }


        private static int Count<T>(IEnumerable<T> values)
        {
            return values == null ? 0 : values.Where(f => f != null).Count();
        }


        private static int Count<T>(ICollection<T> values)
        {
            return values == null ? 0 : values.Where(f => f != null).Count();
        }


        private static bool exists(bool value)
        {
            return value;
        }

        private static string GetValueOrEmpty(bool? val)
        {
            return (val.HasValue) ? val.ToString().ToLower() : string.Empty;
        }

        private static string GetValueOrEmpty(DateTime? val)
        {
            return (val.HasValue) ? val.ToString() : string.Empty;
        }

        private static string GetValueOrEmpty(string val)
        {
            return !string.IsNullOrWhiteSpace(val) ? val : string.Empty;
        }

        private static string GetValueOrEmpty(decimal? val)
        {
            return (val.HasValue) ? val.ToString() : string.Empty;
        }

        private static string GetValueOrEmpty(int? val)
        {
            return (val.HasValue) ? val.ToString() : string.Empty;
        }

        /// <summary>
        /// Get string value between [first] a and [last] b.
        /// </summary>
        public static string Between(string value, string a, string b)
        {
            int posA = value.IndexOf(a);
            int posB = value.LastIndexOf(b);
            if (posA == -1)
            {
                return "";
            }
            if (posB == -1)
            {
                return "";
            }
            int adjustedPosA = posA + a.Length;
            if (adjustedPosA >= posB)
            {
                return "";
            }
            return value.Substring(adjustedPosA, posB - adjustedPosA);
        }

        /// <summary>
        /// Get string value after [first] a.
        /// </summary>
        public static string Before(string value, string a)
        {
            int posA = value.IndexOf(a);
            if (posA == -1)
            {
                return "";
            }
            return value.Substring(0, posA);
        }

        /// <summary>
        /// Get string value after [last] a.
        /// </summary>
        public static string After(string value, string a)
        {
            int posA = value.LastIndexOf(a);
            if (posA == -1)
            {
                return "";
            }
            int adjustedPosA = posA + a.Length;
            if (adjustedPosA >= value.Length)
            {
                return "";
            }
            return value.Substring(adjustedPosA);
        }

        private static int Length(object field)
        {
            if (field == null)
                return 0;
            else
                return field.ToString().Trim().Length;
        }

        private static bool NotSameValues(IEnumerable<object> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return NotSameValues(nodesArray);
        }

        private static bool NotSameValues(params object[] nodes)
        {
            if (nodes == null)
                return false;

            return ((from x in nodes select x).Distinct().Count() == nodes.Count());
        }

        private static bool HasDuplicateValues(IEnumerable<object> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return HasDuplicateValues(nodesArray);
        }

        private static bool HasDuplicateValues(params object[] nodes)
        {
            if (nodes == null)
                return false;

            nodes = nodes.Where(x => x != null).ToArray();
            return !((from x in nodes select x).Distinct().Count() == nodes.Count());

        }

        private int DuplicateValueIndex(IEnumerable<object> values)
        {
            int response = 0;
            var hashset = new HashSet<object>();
            foreach (var value in values)
            {
                if (!hashset.Add(value))
                {
                    return response;
                }
                response++;
            }
            return response;
        }

        private int DuplicateValueIndex<T>(IEnumerable<T?> values) where T : struct
        {
            int response = 0;
            var hashset = new HashSet<T?>();
            foreach (var value in values)
            {
                if (!hashset.Add(value))
                {
                    return response;
                }
                response++;
            }
            return response;
        }

        private static bool IsDate(object value)
        {
            DateTime dateValue;
            return (value != null && DateTime.TryParse(value.ToString(), out dateValue));
        }

        public DateTime AsDate(string dateAsString)
        {
            DateTime response = DateTime.MinValue;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date;
            }

            return response;
        }

        public DateTime Date(DateTime? datetime)
        {
            return datetime.GetValueOrDefault().Date;
        }

        //The logic in After function expects "---" for day and "--" for month. 
        //Since hyphen was missing the date always returned null
        public DateTime? ConvertToDate(int day, int month, int year)
        {
            return ConvertToDate(day == 0 ? null : "---" + day.ToString(), month == 0 ? null : "--" + month.ToString(), year == 0 ? null : year.ToString());
        }

        public DateTime? ConvertToDate(string day, string month, string year)
        {
            DateTime? response;
            DateTime result;

            if (year == null || month == null || day == null)
            {
                return null;
            }
            string dateAsString = year + "-" + After(month, "--") + "-" + After(day, "---");
            if (DateTime.TryParse(dateAsString, out result))
            {
                response = result;
            }
            else
            {
                response = null;
            }
            return response;
        }


        public DateTime? ConvertToDate(string day, string month, int year)
        {
            DateTime? response;
            DateTime result;

            if (year == 0 || month == null || day == null)
            {
                return null;
            }
            string dateAsString = year.ToString() + "-" + After(month, "--") + "-" + After(day, "---");
            if (DateTime.TryParse(dateAsString, out result))
            {
                response = result;
            }
            else
            {
                response = null;
            }
            return response;
        }


        private static int Day(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.Day;
            }

            return response;
        }


        private static int Day(DateTime? date)
        {
            if (date == null)
                return 0;
            else
                return date.Value.Day;

        }


        private static string Month(string dateAsString)
        {
            string response = null;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.ToString("MMMM");
            }
            else
            {
                return "NotAMonth";
            }

            return response;
        }

        private static string Month(DateTime? date)
        {
            if (date == null)
                return "NotAMonth";
            else
                return date.Value.ToString("MMMM");
        }


        private static int MonthAsInt(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.Month;
            }

            return response;
        }

        private static int MonthAsInt(DateTime? date)
        {
            if (date == null)
            {
                return 0;
            }
            return date.Value.Month;
        }


        private static int Year(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.Year;
            }

            return response;
        }

        private static int Year(DateTime? date)
        {
            if (date == null)
                return 0;
            else
                return date.Value.Year;
        }


        private static int CurrentFinancialYear()
        {
            return DateToFinancialYear(DateTime.Now, 7);
        }

        private static int FinancialYear(string dateAsString)
        {
            return DateToFinancialYear(dateAsString, 7);
        }

        private static int FinancialYear(DateTime? date)
        {
            return DateToFinancialYear(date, 7);
        }

        private static int DateToFinancialYear(string dateAsString, int startingMonth)
        {
            int response = 0;
            DateTime date;
            if (DateTime.TryParse(dateAsString, out date))
            {
                response = DateToFinancialYear(date, startingMonth);
            }

            return response;
        }
        private static int DateToFinancialYear(DateTime? date, int startingMonth)
        {
            int response;
            if (date == null)
            {
                response = 0;
            }
            else
            {
                int year = date.Value.Year;
                int month = date.Value.Month;

                if (startingMonth > month)
                    response = year;
                else
                    response = year + 1;
            }
            return response;
        }


        private static int FBTYear(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
                response = FBTYear(date);
            return response;
        }

        private static int FBTYear(DateTime? date)
        {
            if (date == null)
            {
                return 0;
            }
            else
            {
                if (date.Value.Month > 3)
                    return date.Value.Year + 1;
                else
                    return date.Value.Year;
            }
        }

        private static DateTime? AddMonthsToDate(DateTime? dateTime, int months)
        {
            return dateTime == null ? null : (DateTime?)dateTime.Value.AddMonths(months);
        }

        private static bool IsNumeric(object value)
        {
            decimal numbervalue;
            return (value != null && decimal.TryParse(value.ToString(), out numbervalue));
        }

        private static bool NotMonetary(decimal? field, string sign, int digits, int decimals)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;
                string decimalExpression;
                int digitsToUse = digits - decimals;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                if (decimals > 0)
                    decimalExpression = @"(\.\d{1," + decimals + "})?$";
                else
                    decimalExpression = @"$";

                return !(Regex.IsMatch(field.Value.ToString("0.#########################"), signExpression + @"\d{1," + digitsToUse + "}" + decimalExpression));
            }
        }

        private static bool NotNumeric(decimal? field, string sign, int digits, int decimals)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;
                string decimalExpression;
                int digitsToUse = digits - decimals;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                if (decimals > 0)
                    decimalExpression = @"(\.\d{1," + decimals + "})?$";
                else
                    decimalExpression = @"$";

                return !(Regex.IsMatch(field.Value.ToString("0.#########################"), signExpression + @"\d{1," + digitsToUse + "}" + decimalExpression));
            }
        }

        private static bool NotNumeric(int? field, string sign, int digits, int decimals = 0)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                return !(Regex.IsMatch(field.Value.ToString(), signExpression + @"\d{1," + digits + "}$"));
            }
        }

        private static bool NotNumeric(long? field, string sign, int digits, int decimals = 0)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                return !(Regex.IsMatch(field.Value.ToString(), signExpression + @"\d{1," + digits + "}$"));
            }
        }


        private static bool OutsideRange(decimal field, decimal expression, int range)
        {
            bool response;

            response = (field < (expression - range)) || (field > (expression + range));

            return response;
        }


        private static bool FailsUSIAlgorithm(string usi, string abn)
        {
            bool response = true;
            if (usi == null || abn == null)
            {
                response = false;
            }
            else
            {
                usi = usi.Trim();
                abn = abn.Trim();
                if (usi.Length == 14)
                {
                    int numeric;
                    if (usi.Substring(0, 11) == abn && int.TryParse(usi.Substring(11, 3), out numeric))
                        response = false;

                }
                else if (usi.Length == 9)
                {

                    if (Regex.IsMatch(usi, @"^([a-zA-Z]{3}\d{4}[a-zA-Z]{2})$"))
                        response = false;
                }
            }
            return response;
        }

        private static bool FailsTANAlgorithm(string tan)
        {
            bool response;
            decimal decimalTan;

            if (tan == null)
                return false;

            tan = tan.Trim();

            if (!decimal.TryParse(tan, out decimalTan))
                return true;

            if (tan.Length != 8)
                return true;

            decimal tanSum =
                7 * int.Parse(tan.Substring(0, 1)) +
                9 * int.Parse(tan.Substring(1, 1)) +
                8 * int.Parse(tan.Substring(2, 1)) +
                4 * int.Parse(tan.Substring(3, 1)) +
                6 * int.Parse(tan.Substring(4, 1)) +
                3 * int.Parse(tan.Substring(5, 1)) +
                5 * int.Parse(tan.Substring(6, 1)) +
                1 * int.Parse(tan.Substring(7, 1));

            if ((tanSum % 11) == 0)
                response = false;
            else
                response = true;

            return response;
        }


        private static bool FailsABNAlgorithm(string abn)
        {
            bool response;
            decimal decimalAbn;

            if (abn == null)
                return false;

            abn = abn.Trim();

            if (!decimal.TryParse(abn, out decimalAbn))
                return true;

            if (abn.Length != 11)
                return true;

            decimal abnSum =
                10 * (int.Parse(abn.Substring(0, 1)) - 1) +
                1 * int.Parse(abn.Substring(1, 1)) +
                3 * int.Parse(abn.Substring(2, 1)) +
                5 * int.Parse(abn.Substring(3, 1)) +
                7 * int.Parse(abn.Substring(4, 1)) +
                9 * int.Parse(abn.Substring(5, 1)) +
                11 * int.Parse(abn.Substring(6, 1)) +
                13 * int.Parse(abn.Substring(7, 1)) +
                15 * int.Parse(abn.Substring(8, 1)) +
                17 * int.Parse(abn.Substring(9, 1)) +
                19 * int.Parse(abn.Substring(10, 1));

            if ((abnSum % 89) == 0)
                response = false;
            else
                response = true;

            return response;
        }
        private static bool FailsACNAlgorithm(string acn)
        {
            bool response;
            decimal decimalAbn;
            if (acn == null)
                return false;

            acn = acn.Trim();

            if (!decimal.TryParse(acn, out decimalAbn))
                return true;

            if (acn.Length != 9)
                return true;

            decimal abnSum =
                8 * int.Parse(acn.Substring(0, 1)) +
                7 * int.Parse(acn.Substring(1, 1)) +
                6 * int.Parse(acn.Substring(2, 1)) +
                5 * int.Parse(acn.Substring(3, 1)) +
                4 * int.Parse(acn.Substring(4, 1)) +
                3 * int.Parse(acn.Substring(5, 1)) +
                2 * int.Parse(acn.Substring(6, 1)) +
                1 * int.Parse(acn.Substring(7, 1));

            decimal checkDigit = int.Parse(acn.Substring(8, 1));
            decimal acnRemainder = abnSum % 10;

            if (((10 - acnRemainder) % 10) == checkDigit)
                response = false;
            else
                response = true;

            return response;
        }


        private static bool FailsTFNAlgorithm(string tfn)
        {
            bool response;
            decimal decimalTfn;

            if (tfn == null)
                return false;

            tfn = tfn.Trim();
            tfn = Regex.Replace(tfn, "^0+", "");

            if (!decimal.TryParse(tfn, out decimalTfn))
                return true;

            if (tfn.Length < 8)
                return true;


            decimal tfn1To7Sum =
                1 * int.Parse(tfn.Substring(0, 1)) +
                4 * int.Parse(tfn.Substring(1, 1)) +
                3 * int.Parse(tfn.Substring(2, 1)) +
                7 * int.Parse(tfn.Substring(3, 1)) +
                5 * int.Parse(tfn.Substring(4, 1)) +
                8 * int.Parse(tfn.Substring(5, 1)) +
                6 * int.Parse(tfn.Substring(6, 1));

            decimal tfn8 = 9 * int.Parse(tfn.Substring(7, 1));

            if (tfn.Length == 8)
            {
                decimal tFNLg8WSum9 = 10 * int.Parse(tfn.Substring(7, 1));
                decimal tFNLg8WSum = tfn1To7Sum + tFNLg8WSum9;

                if ((tFNLg8WSum % 11) == 0)
                    response = false;
                else
                    response = true;
            }
            else if (tfn.Length == 9)
            {
                decimal tfn9 = 10 * int.Parse(tfn.Substring(8, 1));
                decimal tFNLg9WSum = tfn1To7Sum + tfn8 + tfn9;

                if ((tFNLg9WSum % 11) == 0)
                    response = false;
                else
                    response = true;
            }
            else
            {
                response = true;
            }

            return response;
        }


        private static decimal ConditionalValue(bool expression, decimal? trueVal, decimal? falseVal)
        {
            return expression ? trueVal.GetValueOrDefault() : falseVal.GetValueOrDefault();
        }

        private static decimal AsNumeric(string value)
        {
            decimal numberValue;
            decimal.TryParse(value, out numberValue);
            return numberValue;
        }

        private static bool RegexMatch(int? field, string expression, string flags = "")
        {
            return IsMatch(field, expression, GetRegexOption(flags));
        }

        private static bool RegexMatch(string field, string expression, string flags = "")
        {
            return IsMatch(field, expression, GetRegexOption(flags));
        }

        private static RegexOptions GetRegexOption(string flags)
        {
            RegexOptions options = RegexOptions.None;

            char[] characters = flags.ToCharArray();

            foreach (char character in characters)
            {
                switch (character)
                {
                    case 'i':
                        options = options | RegexOptions.IgnoreCase;
                        break;
                    case 'm':
                        options = options | RegexOptions.Multiline;
                        break;
                    case 's':
                        options = options | RegexOptions.Singleline;
                        break;
                    case 'n':
                        options = options | RegexOptions.ExplicitCapture;
                        break;
                    case 'x':
                        options = options | RegexOptions.IgnorePatternWhitespace;
                        break;
                }
            }

            return options;
        }

        /// <summary>
        /// Returns an occurrence index as [occurrenceIndex] of occurrenceIndex > 0, otherwise the empty string
        /// </summary>
        /// <param name="occurrenceIndex">Index of the occurrence.</param>
        /// <returns>Occurrence in XPath [#] format</returns>
        public string OccurrenceIndex(int occurrenceIndex)
        {
            return occurrenceIndex > 0 ? "[" + occurrenceIndex + "]" : "";
        }

        #endregion // Functions

        public DDCTNS2019 ConsumedReport { get; private set; }

        private static Dictionary<string, ProcessMessageDocument> _processMessageDocuments = new Dictionary<string, ProcessMessageDocument>();

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public List<ProcessMessageDocument> ValidateReport(DDCTNS2019 report)
        {

            List<ProcessMessageDocument> response = new List<ProcessMessageDocument>();
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            bool assertion;

            this.ConsumedReport = report;

            #region Manual Rules - Rules marked for manual coding

            #region VR.ATO.DDCTNS.000151

            /*  VR.ATO.DDCTNS.000151
                        Entity identifier scheme for ReportingParty must be "http://www.ato.gov.au/tfn"

                    Legacy Rule Format:
                        (^Context <> NULL AND ^IdentifierScheme <> 'http://www.ato.gov.au/tfn')

                Technical Business Rule Format:
                        ^Context <> NULL AND ^IdentifierScheme <> 'http://www.ato.gov.au/tfn'

            Data Elements:

                ^Context = Context

                ^IdentifierScheme = IdentifierScheme
                */
            VRATODDCTNS000151(response, report);
            #endregion // VR.ATO.DDCTNS.000151

            #region VR.ATO.DDCTNS.000152

            /*  VR.ATO.DDCTNS.000152
                        TFN in entity identifier of context does not match the TFN provided for the Reporting Party

                    Legacy Rule Format:
                        (^Context <> NULL AND ^TFN <> ^DDCTNS302)

                Technical Business Rule Format:
                        ^Context <> NULL AND ^TFN <> ^DDCTNS302

            Data Elements:

                ^Context = Context

                ^DDCTNS302 = RP:Identifiers.TaxFileNumber.Identifier

                ^TFN = TFN
                */
            VRATODDCTNS000152(response, report);
            #endregion // VR.ATO.DDCTNS.000152

            #region VR.ATO.DDCTNS.000153

            /*  VR.ATO.DDCTNS.000153
                        Invalid context. Period start date must be the same across all duration contexts.

                    Legacy Rule Format:
                        (^Context <> NULL AND ^period.startDate <> ^DDCTNS300)

                Technical Business Rule Format:
                        ^Context <> NULL AND ^period.startDate <> ^DDCTNS300

            Data Elements:

                ^Context = Context

                ^DDCTNS300 = RP:Lodgment.PeriodStart.Date

                ^period.startDate = period.startDate
                */
            VRATODDCTNS000153(response, report);
            #endregion // VR.ATO.DDCTNS.000153

            #region VR.ATO.DDCTNS.000154

            /*  VR.ATO.DDCTNS.000154
                        Invalid context. Period end date must be the same across all duration contexts.

                    Legacy Rule Format:
                        (^Context <> NULL AND ^period.endDate <> ^DDCTNS301)

                Technical Business Rule Format:
                        ^Context <> NULL AND ^period.endDate <> ^DDCTNS301

            Data Elements:

                ^Context = Context

                ^DDCTNS301 = RP:Lodgment.PeriodEnd.Date

                ^period.endDate = period.endDate
                */
            VRATODDCTNS000154(response, report);
            #endregion // VR.ATO.DDCTNS.000154

            #region VR.ATO.DDCTNS.000155

            /*  VR.ATO.DDCTNS.000155
                        At least one deduction detail must be provided

                    Legacy Rule Format:
                        (^DDCTNS200 = NULL AND ^DDCTNS310 = NULL AND ^DDCTNS311 = NULL AND ^DDCTNS312 = NULL AND ^DDCTNS417 = NULL AND
                        Count(^RP.{CarSeqNum}) = 0 AND Count(^RP.{EduSeqNum}) = 0 AND Count(^RP.{MangTaxAffSeqNum}) = 0 AND Count(^RP.{SCSeqNum}) = 0 AND Count(^RP.{TrvlSeqNum}) = 0 AND Count(^RP.{ProjPoolSeqNum}) = 0 AND Count(^RP.{ForestManInvSeqNum}) = 0 AND Count(^RP.{GiftDonSeqNum}) = 0 AND Count(^RP.{IntDedSeqNum}) = 0 AND Count(^RP.{DivDedSeqNum}) = 0 AND Count(^RP.{OTHWRSeqNum}) = 0 AND Count(^RP.{ClthngSeqNum}) = 0)

                Technical Business Rule Format:
                        (^DDCTNS200 = NULL AND ^DDCTNS310 = NULL AND ^DDCTNS311 = NULL AND ^DDCTNS312 = NULL AND ^DDCTNS417 = NULL AND
                        Count(^RP.{CarSeqNum}) = 0 AND Count(^RP.{EduSeqNum}) = 0 AND Count(^RP.{MangTaxAffSeqNum}) = 0 AND Count(^RP.{SCSeqNum}) = 0 AND Count(^RP.{TrvlSeqNum}) = 0 AND Count(^RP.{ProjPoolSeqNum}) = 0 AND Count(^RP.{ForestManInvSeqNum}) = 0 AND Count(^RP.{GiftDonSeqNum}) = 0 AND Count(^RP.{IntDedSeqNum}) = 0 AND Count(^RP.{DivDedSeqNum}) = 0 AND Count(^RP.{OTHWRSeqNum}) = 0 AND Count(^RP.{ClthngSeqNum}) = 0)

            Data Elements:

                ^DDCTNS200 = RP:IncomeTax.Deduction.EducationCircumstances.Code

                ^DDCTNS310 = RP:IncomeTax.Deduction.LowValuePoolFinancialInvestments.Amount

                ^DDCTNS311 = RP:IncomeTax.Deduction.LowValuePoolRentalProperties.Amount

                ^DDCTNS312 = RP:IncomeTax.Deduction.LowValuePoolOther.Amount

                ^DDCTNS417 = RP:IncomeTax.Deduction.Election.Amount
                */
            VRATODDCTNS000155(response, report);
            #endregion // VR.ATO.DDCTNS.000155
            #endregion Manual Rules - Rules marked for manual coding

            #region VR.ATO.DDCTNS.000001

            /*  VR.ATO.DDCTNS.000001
            The Sequence Container within the Car Sequence Dimension context instance must not be duplicated
    
            Legacy Rule Format:
            (HasDuplicateValues(^DDCTNS100))

            Technical Business Rule Format:
            (HasDuplicateValues(^DDCTNS100))
    
            Data Elements:
    
            ^DDCTNS100 = RP.{CarSeqNum}:Report.ItemOrder.Number
            */
            assertion = HasDuplicateValues(report.RPCarSeqNumContextCollection == null ? null : report.RPCarSeqNumContextCollection.Select(f => f.DDCTNS100).Cast<object>().ToArray());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000001", 
					Severity = ProcessMessageSeverity.Error,
                    Description = @"Sequence Container must not be duplicated",
                    LongDescription = @"The Sequence Container within the Car Sequence Dimension context instance must not be duplicated",
                    Location = "/xbrli:xbrl/xbrli:context[@id='" + report.RPCarSeqNumContextCollection[DuplicateValueIndex(report.RPCarSeqNumContextCollection.Select(f => f.DDCTNS100))].Id + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000001" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000001

            #region VR.ATO.DDCTNS.000002

            /*  VR.ATO.DDCTNS.000002
            The number of Reporting Party Car contexts must not exceed 10.
    
            Legacy Rule Format:
            (Count(RP.{CarSeqNum}) > 10)
            ENDIF

            Technical Business Rule Format:
            (Count(^RP.{CarSeqNum}:Context) > 10)
    
            Data Elements:
            */
            assertion = (Count(report.RPCarSeqNumContextCollection) > 10);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000002",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting Party Car contexts must not exceed 10.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPCarSeqNumContextCollection == null ? 0 : (report.RPCarSeqNumContextCollection.Count() == 0 ? 0 : report.RPCarSeqNumContextCollection.Last().LastOccurrenceIndex)),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000002" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000002

            #region VR.ATO.DDCTNS.W00007

            /*  VR.ATO.DDCTNS.W00007
            The total number of all Car expenses business kilometres travelled exceeds 10,000. Check and correct if necessary.
    
            Legacy Rule Format:
            (Sum(^DDCTNS103) > 10000)

            Technical Business Rule Format:
            (Sum(^DDCTNS103) > 10000)
    
            Data Elements:
    
            ^DDCTNS103 = RP.{CarSeqNum}:IncomeTax.Deduction.CarBusinessKilometres.Number
            */
            assertion = ((report.RPCarSeqNumContextCollection == null ? 0 : report.RPCarSeqNumContextCollection.Sum(f => f.DDCTNS103.GetValueOrDefault())) > 10000);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.W00007",
                    Severity = ProcessMessageSeverity.Warning,
                    Description = @"Check total number of Car expenses business kilometres travelled",
                    LongDescription = @"The total number of all Car expenses business kilometres travelled exceeds 10,000. Check and correct if necessary.",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.CarBusinessKilometres.Number[@contextRef='" + report.RPCarSeqNumContextCollection.First().Id + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.W00007" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "DDCTNS103", Value = "DDCTNS103" });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.W00007

            #region VR.ATO.DDCTNS.000016

            /*  VR.ATO.DDCTNS.000016
            The number of Reporting Party Dividend Deductions contexts must not exceed 20.
    
            Legacy Rule Format:
            (Count(RP.{DivDedSeqNum}) > 20)

            Technical Business Rule Format:
            (Count(^RP.{DivDedSeqNum}:Context) > 20)
    
            Data Elements:
            */
            assertion = (Count(report.RPDivDedSeqNumContextCollection) > 20);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000016",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting Party Dividend Deductions contexts must not exceed 20.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPDivDedSeqNumContextCollection == null ? 0 : (report.RPDivDedSeqNumContextCollection.Count() == 0 ? 0 : report.RPDivDedSeqNumContextCollection.Last().LastOccurrenceIndex)),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000016" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000016

            #region VR.ATO.DDCTNS.000017

            /*  VR.ATO.DDCTNS.000017
            The Sequence Container within the Dividend Deductions Sequence Dimension context instance must not be duplicated
    
            Legacy Rule Format:
            (HasDuplicateValues(^DDCTNS130))

            Technical Business Rule Format:
            (HasDuplicateValues(^DDCTNS130))
    
            Data Elements:
    
            ^DDCTNS130 = RP.{DivDedSeqNum}:Report.ItemOrder.Number
            */
            assertion = HasDuplicateValues(report.RPDivDedSeqNumContextCollection == null ? null : report.RPDivDedSeqNumContextCollection.Select(f => f.DDCTNS130).Cast<object>().ToArray());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000017",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Sequence Container must not be duplicated",
                    LongDescription = @"The Sequence Container within the Dividend Deductions Sequence Dimension context instance must not be duplicated",
                    Location = "/xbrli:xbrl/xbrli:context[@id='" + report.RPDivDedSeqNumContextCollection[DuplicateValueIndex(report.RPDivDedSeqNumContextCollection.Select(f => f.DDCTNS130))].Id + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000017" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000017

            #region VR.ATO.DDCTNS.000018

            /*  VR.ATO.DDCTNS.000018
            Where a Reporting Party Dividend Deductions instance is present, Dividend deduction amount must be provided
    
            Legacy Rule Format:
            (AnyOccurrence(^DDCTNS132, ^DDCTNS132 = NULL))

            Technical Business Rule Format:
            (AnyOccurrence(^DDCTNS132, ^DDCTNS132 = NULL))
    
            Data Elements:
    
            ^DDCTNS132 = RP.{DivDedSeqNum}:IncomeTax.Deduction.Dividend.Amount
            */
            assertion = (report.RPDivDedSeqNumContextCollection == null ? false : report.RPDivDedSeqNumContextCollection.Any(DDCTNS132Repeat => DDCTNS132Repeat.DDCTNS132 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000018",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Dividend deduction amount must be provided",
                    LongDescription = @"Where a Reporting Party Dividend Deductions instance is present, Dividend deduction amount must be provided",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.Dividend.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000018" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "DDCTNS132", Value = "DDCTNS132" });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000018
			
			#region VR.ATO.DDCTNS.000019
    
            /*  VR.ATO.DDCTNS.000019
            The number of Reporting Party Other Deductions contexts must not exceed 100.
    
            Legacy Rule Format:
            (Count(RP.{OthDedSeqNum}) > 100)

            Technical Business Rule Format:
            (Count(^RP.{OthDedSeqNum}:Context) > 100)
    
            Data Elements:
            */
            assertion = (Count(report.RPOthDedSeqNumContextCollection) > 100);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000019", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting Party Other Deductions contexts must not exceed 100.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOthDedSeqNumContextCollection == null ? 0 : (report.RPOthDedSeqNumContextCollection.Count() == 0 ? 0 : report.RPOthDedSeqNumContextCollection.Last().LastOccurrenceIndex)),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000019"} },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000019
			
			#region VR.ATO.DDCTNS.000020
    
            /*  VR.ATO.DDCTNS.000020
            The Sequence Container within the Other Deductions Sequence Dimension context instance must not be duplicated
    
            Legacy Rule Format:
            (HasDuplicateValues(^DDCTNS135))

            Technical Business Rule Format:
            (HasDuplicateValues(^DDCTNS135))
    
            Data Elements:
    
            ^DDCTNS135 = RP.{OthDedSeqNum}:Report.ItemOrder.Number
            */
            assertion = HasDuplicateValues(report.RPOthDedSeqNumContextCollection == null ? null : report.RPOthDedSeqNumContextCollection.Select(f => f.DDCTNS135).Cast<object>().ToArray());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000020", Severity = ProcessMessageSeverity.Error,
                    Description = @"Sequence Container must not be duplicated",
                    LongDescription = @"The Sequence Container within the Other Deductions Sequence Dimension context instance must not be duplicated",
                    Location = "/xbrli:xbrl/xbrli:context[@id='" + report.RPOthDedSeqNumContextCollection[DuplicateValueIndex(report.RPOthDedSeqNumContextCollection.Select(f => f.DDCTNS135))].Id + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000020"} },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000020
    
            #region VR.ATO.DDCTNS.000021
    
            /*  VR.ATO.DDCTNS.000021
            Where a Reporting Party Other Deductions instance is present, Other deduction claim type code must be provided
    
            Legacy Rule Format:
            (AnyOccurrence(^DDCTNS136, ^DDCTNS136 = NULL))


            Technical Business Rule Format:
            (AnyOccurrence(^DDCTNS136, ^DDCTNS136 = NULL))
    
            Data Elements:
    
            ^DDCTNS136 = RP.{OthDedSeqNum}:Expense.DeductibleOther.Code
            */
            assertion = (report.RPOthDedSeqNumContextCollection == null ? false : report.RPOthDedSeqNumContextCollection.Any(DDCTNS136Repeat => DDCTNS136Repeat.DDCTNS136 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000021", Severity = ProcessMessageSeverity.Error,
                    Description = @"Other deduction claim type code must be provided",
                    LongDescription = @"Where a Reporting Party Other Deductions instance is present, Other deduction claim type code must be provided",
                    Location = "/xbrli:xbrl/tns:Expense.DeductibleOther.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000021"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "DDCTNS136", Value = "DDCTNS136" });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000021
    
            #region VR.ATO.DDCTNS.000022
    
            /*  VR.ATO.DDCTNS.000022
            Where a Reporting Party Other Deductions instance is present, Other deductions amount must be provided
    
            Legacy Rule Format:
            (AnyOccurrence(^DDCTNS138, ^DDCTNS138 = NULL))


            Technical Business Rule Format:
            (AnyOccurrence(^DDCTNS138, ^DDCTNS138 = NULL))
    
            Data Elements:
    
            ^DDCTNS138 = RP.{OthDedSeqNum}:Expense.DeductibleOther.Amount
            */
            assertion = (report.RPOthDedSeqNumContextCollection == null ? false : report.RPOthDedSeqNumContextCollection.Any(DDCTNS138Repeat => DDCTNS138Repeat.DDCTNS138 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000022", Severity = ProcessMessageSeverity.Error,
                    Description = @"Other deductions amount must be provided",
                    LongDescription = @"Where a Reporting Party Other Deductions instance is present, Other deductions amount must be provided",
                    Location = "/xbrli:xbrl/tns:Expense.DeductibleOther.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000022"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "DDCTNS138", Value = "DDCTNS138" });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000022
            #region VR.ATO.DDCTNS.000200

            /*  VR.ATO.DDCTNS.000200
            There must not be more than 20 Cost of Managing Tax Affairs contexts

            #region VR.ATO.DDCTNS.000100

            /*  VR.ATO.DDCTNS.000100
            When Self education expense instances are present, the Self education reason must be provided
    
            Legacy Rule Format:
            (Count(RP.{MangTaxAffSeqNum}) > 20)
            IF ^DDCTNS200 = NULL AND Count(RP.{EduSeqNum}) > 0
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^DDCTNS200 = NULL AND Count(^Context) > 0
    
            Data Elements:
    
            ^DDCTNS200 = RP:IncomeTax.Deduction.EducationCircumstances.Code
    
            ^Context = RP:RP.{EduSeqNum}
            */
            assertion = (report.DDCTNS200 == null && Count(report.RPEduSeqNumContextCollection) > 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000100",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Self education reason must be provided",
                    LongDescription = @"When Self education expense instances are present, the Self education reason must be provided",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.EducationCircumstances.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000100" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "DDCTNS200", Value = GetValueOrEmpty(report.DDCTNS200) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000100

            #region VR.ATO.DDCTNS.000101

            /*  VR.ATO.DDCTNS.000101
            The number of Reporting Party Education contexts must not exceed 20
    
            Legacy Rule Format:
            IF Count(RP.{EduSeqNum}) > 20
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 20
            (Count(^RP.{MangTaxAffSeqNum}:Context) > 20)
    
            Data Elements:
            */
            assertion = (Count(report.RPEduSeqNumContextCollection) > 20);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000101",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting Party Education contexts must not exceed 20.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPEduSeqNumContextCollection == null ? 0 : (report.RPEduSeqNumContextCollection.Count() == 0 ? 0 : report.RPEduSeqNumContextCollection.Last().LastOccurrenceIndex)),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000101" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000101

            #region VR.ATO.DDCTNS.000102

            /*  VR.ATO.DDCTNS.000102
            The Sequence Container within the Education Sequence Dimension context instance must not be duplicated
    
            Legacy Rule Format:
            (HasDuplicateValues(^DDCTNS201))

            Technical Business Rule Format:
            (HasDuplicateValues(^DDCTNS201))
    
            Data Elements:
    
            ^DDCTNS201 = RP.{EduSeqNum}:Report.ItemOrder.Number
            */
            assertion = HasDuplicateValues(report.RPEduSeqNumContextCollection == null ? null : report.RPEduSeqNumContextCollection.Select(f => f.DDCTNS201).Cast<object>().ToArray());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000102",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Sequence Container must not be duplicated",
                    LongDescription = @"The Sequence Container within the Education Sequence Dimension context instance must not be duplicated",
                    Location = "/xbrli:xbrl/xbrli:context[@id='" + report.RPEduSeqNumContextCollection[DuplicateValueIndex(report.RPEduSeqNumContextCollection.Select(f => f.DDCTNS201))].Id + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000102" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000102

            #region VR.ATO.DDCTNS.000103

            /*  VR.ATO.DDCTNS.000103
            Where a Reporting Party Self education instance is present, Self education expense category must be provided
    
            Legacy Rule Format:
            (AnyOccurrence(^DDCTNS202, ^DDCTNS202 = NULL))

            Technical Business Rule Format:
            (AnyOccurrence(^DDCTNS202, ^DDCTNS202 = NULL))
    
            Data Elements:
    
            ^DDCTNS202 = RP.{EduSeqNum}:IncomeTax.Deduction.SelfEducation.Code
            */
            assertion = (report.RPEduSeqNumContextCollection == null ? false : report.RPEduSeqNumContextCollection.Any(DDCTNS202Repeat => DDCTNS202Repeat.DDCTNS202 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000103",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Self education expense category must be provided",
                    LongDescription = @"Where a Reporting Party Self education instance is present, Self education expense category must be provided",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.SelfEducation.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000103" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "DDCTNS202", Value = "DDCTNS202" });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000103

            #region VR.ATO.DDCTNS.000107

            /*  VR.ATO.DDCTNS.000107
            The number of Reporting Party Project Pool contexts must not exceed 20
    
            Legacy Rule Format:
            IF Count(RP.{ProjPoolSeqNum}) > 20
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 20
    
            Data Elements:
    
            ^Context = RP:RP.{ProjPoolSeqNum}
            */
            assertion = (Count(report.RPProjPoolSeqNumContextCollection) > 20);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000107",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting Party Project Pool contexts must not exceed 20.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPProjPoolSeqNumContextCollection == null ? 0 : (report.RPProjPoolSeqNumContextCollection.Count() == 0 ? 0 : report.RPProjPoolSeqNumContextCollection.Last().LastOccurrenceIndex)),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000107" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000107

            #region VR.ATO.DDCTNS.000108

            /*  VR.ATO.DDCTNS.000108
            The Sequence Container within the Project Pool Sequence Dimension context instance must not be duplicated
    
            Legacy Rule Format:
            (HasDuplicateValues(^DDCTNS210))

            Technical Business Rule Format:
            (HasDuplicateValues(^DDCTNS210))
    
            Data Elements:
    
            ^DDCTNS210 = RP.{ProjPoolSeqNum}:Report.ItemOrder.Number
            */
            assertion = HasDuplicateValues(report.RPProjPoolSeqNumContextCollection == null ? null : report.RPProjPoolSeqNumContextCollection.Select(f => f.DDCTNS210).Cast<object>().ToArray());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000108",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Sequence Container must not be duplicated",
                    LongDescription = @"The Sequence Container within the Project Pool Sequence Dimension context instance must not be duplicated",
                    Location = "/xbrli:xbrl/xbrli:context[@id='" + report.RPProjPoolSeqNumContextCollection[DuplicateValueIndex(report.RPProjPoolSeqNumContextCollection.Select(f => f.DDCTNS210))].Id + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000108" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000108

            #region VR.ATO.DDCTNS.000109

            /*  VR.ATO.DDCTNS.000109
            Where a Reporting Party Project pool instance is present, Project pool amount must be provided
    
            Legacy Rule Format:
            (AnyOccurrence(^DDCTNS212, ^DDCTNS212 = NULL))

            Technical Business Rule Format:
            (AnyOccurrence(^DDCTNS212, ^DDCTNS212 = NULL))
    
            Data Elements:
    
            ^DDCTNS212 = RP.{ProjPoolSeqNum}:IncomeTax.Deduction.ProjectPool.Amount
            */
            assertion = (report.RPProjPoolSeqNumContextCollection == null ? false : report.RPProjPoolSeqNumContextCollection.Any(DDCTNS212Repeat => DDCTNS212Repeat.DDCTNS212 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000109",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Project pool amount must be provided",
                    LongDescription = @"Where a Reporting Party Project pool instance is present, Project pool amount must be provided",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.ProjectPool.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000109" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "DDCTNS212", Value = "DDCTNS212" });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000109

            #region VR.ATO.DDCTNS.000200

            /*  VR.ATO.DDCTNS.000200
            There must not be more than 20 Cost of Managing Tax Affairs contexts
    
            Legacy Rule Format:
            (Count(RP.{MangTaxAffSeqNum}) > 20)

            Technical Business Rule Format:
            (Count(^RP.{MangTaxAffSeqNum}:Context) > 20)
    
            Data Elements:
            */
            assertion = (Count(report.RPMangTaxAffSeqNumContextCollection) > 20);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000200", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Cost of Managing Tax Affairs contexts must not exceed 20.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPMangTaxAffSeqNumContextCollection == null ? 0 : (report.RPMangTaxAffSeqNumContextCollection.Count() == 0 ? 0 : report.RPMangTaxAffSeqNumContextCollection.Last().LastOccurrenceIndex)),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000200" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000200

            #region VR.ATO.DDCTNS.000201

            /*  VR.ATO.DDCTNS.000201
            The Sequence Container within the Cost of managing tax affairs Sequence Dimension context instance must not be duplicated
    
            Legacy Rule Format:
            (HasDuplicateValues(^DDCTNS303))

            Technical Business Rule Format:
            (HasDuplicateValues(^DDCTNS303))
    
            Data Elements:
    
            ^DDCTNS303 = RP.{MangTaxAffSeqNum}:Report.ItemOrder.Number
            */
            assertion = HasDuplicateValues(report.RPMangTaxAffSeqNumContextCollection == null ? null : report.RPMangTaxAffSeqNumContextCollection.Select(f => f.DDCTNS303).Cast<object>().ToArray());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000201",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Sequence Container must not be duplicated",
                    LongDescription = @"The Sequence Container within the Cost of managing tax affairs Sequence Dimension context instance must not be duplicated",
                    Location = "/xbrli:xbrl/xbrli:context[@id='" + report.RPMangTaxAffSeqNumContextCollection[DuplicateValueIndex(report.RPMangTaxAffSeqNumContextCollection.Select(f => f.DDCTNS303))].Id + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000201" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000201

            #region VR.ATO.DDCTNS.000206

            /*  VR.ATO.DDCTNS.000206
            There must not be more than 20 forestry managed investment contexts
    
            Legacy Rule Format:
            (Count(RP.{ForestManInvSeqNum}) > 20)

            Technical Business Rule Format:
            (Count(^RP.{ForestManInvSeqNum}:Context) > 20)
    
            Data Elements:
            */
            assertion = (Count(report.RPForestManInvSeqNumContextCollection) > 20);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000206",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Forestry Managed Investment contexts must not exceed 20.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPForestManInvSeqNumContextCollection == null ? 0 : (report.RPForestManInvSeqNumContextCollection.Count() == 0 ? 0 : report.RPForestManInvSeqNumContextCollection.Last().LastOccurrenceIndex)),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000206" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000206

            #region VR.ATO.DDCTNS.000207

            /*  VR.ATO.DDCTNS.000207
            The Sequence Container within the forestry managed investment Sequence Dimension context instance must not be duplicated
    
            Legacy Rule Format:
            (HasDuplicateValues(^DDCTNS307))

            Technical Business Rule Format:
            (HasDuplicateValues(^DDCTNS307))
    
            Data Elements:
    
            ^DDCTNS307 = RP.{ForestManInvSeqNum}:Report.ItemOrder.Number
            */
            assertion = HasDuplicateValues(report.RPForestManInvSeqNumContextCollection == null ? null : report.RPForestManInvSeqNumContextCollection.Select(f => f.DDCTNS307).Cast<object>().ToArray());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000207",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Sequence Container must not be duplicated",
                    LongDescription = @"The Sequence Container within the Forestry Managed Investment Sequence Dimension context instance must not be duplicated",
                    Location = "/xbrli:xbrl/xbrli:context[@id='" + report.RPForestManInvSeqNumContextCollection[DuplicateValueIndex(report.RPForestManInvSeqNumContextCollection.Select(f => f.DDCTNS307))].Id + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000207" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000207

            #region VR.ATO.DDCTNS.000208

            /*  VR.ATO.DDCTNS.000208
            Where a Reporting Party Cost of managing tax affairs instance is present, Cost of managing tax affairs type code must be provided
    
            Legacy Rule Format:
            (AnyOccurrence(^DDCTNS304, ^DDCTNS304 = NULL))

            Technical Business Rule Format:
            (AnyOccurrence(^DDCTNS304, ^DDCTNS304 = NULL))
    
            Data Elements:
    
            ^DDCTNS304 = RP.{MangTaxAffSeqNum}:IncomeTax.Deduction.TaxAffairManagementAdjustmentReason.Code
            */
            assertion = (report.RPMangTaxAffSeqNumContextCollection == null ? false : report.RPMangTaxAffSeqNumContextCollection.Any(DDCTNS304Repeat => DDCTNS304Repeat.DDCTNS304 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000208",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Cost of managing tax affairs type code must be provided",
                    LongDescription = @"Where a Reporting Party Cost of managing tax affairs instance is present, Cost of managing tax affairs type code must be provided",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TaxAffairManagementAdjustmentReason.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000208" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "DDCTNS304", Value = "DDCTNS304"
                });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000208

            #region VR.ATO.DDCTNS.000209

            /*  VR.ATO.DDCTNS.000209
            Where a Reporting Party Cost of managing tax affairs instance is present, Cost of managing tax affairs amount must be provided
    
            Legacy Rule Format:
            (AnyOccurrence(^DDCTNS306, ^DDCTNS306 = NULL))

            Technical Business Rule Format:
            (AnyOccurrence(^DDCTNS306, ^DDCTNS306 = NULL))
    
            Data Elements:
    
            ^DDCTNS306 = RP.{MangTaxAffSeqNum}:IncomeTax.Deduction.TaxAffairManagement.Amount
            */
            assertion = (report.RPMangTaxAffSeqNumContextCollection == null ? false : report.RPMangTaxAffSeqNumContextCollection.Any(DDCTNS306Repeat => DDCTNS306Repeat.DDCTNS306 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000209",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Cost of managing tax affairs amount must be provided",
                    LongDescription = @"Where a Reporting Party Cost of managing tax affairs instance is present, Cost of managing tax affairs amount must be provided",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TaxAffairManagement.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000209" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "DDCTNS306", Value = "DDCTNS306"
                });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000209

            #region VR.ATO.DDCTNS.000210

            /*  VR.ATO.DDCTNS.000210
            Where a Reporting Party forestry managed investment instance is present, Forestry managed amount must be provided
    
            Legacy Rule Format:
            (AnyOccurrence(^DDCTNS309, ^DDCTNS309 = NULL))

            Technical Business Rule Format:
            (AnyOccurrence(^DDCTNS309, ^DDCTNS309 = NULL))
    
            Data Elements:
    
            ^DDCTNS309 = RP.{ForestManInvSeqNum}:Expense.ForestryManagedInvestmentSchemeDeduction.Amount
            */
            assertion = (report.RPForestManInvSeqNumContextCollection == null ? false : report.RPForestManInvSeqNumContextCollection.Any(DDCTNS309Repeat => DDCTNS309Repeat.DDCTNS309 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000210",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Forestry managed investment amount must be provided",
                    LongDescription = @"Where a Reporting Party forestry managed investment instance is present, Forestry managed amount must be provided",
                    Location = "/xbrli:xbrl/tns:Expense.ForestryManagedInvestmentSchemeDeduction.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000210" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "DDCTNS309", Value = "DDCTNS309" });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000210

            #region VR.ATO.DDCTNS.000211

            /*  VR.ATO.DDCTNS.000211
            If any low value pool amount is provided, all must be provided
    
            Legacy Rule Format:
            ((^DDCTNS310 <> NULL OR ^DDCTNS311 <> NULL OR ^DDCTNS312 <> NULL) AND (^DDCTNS310 = NULL OR ^DDCTNS311 = NULL OR ^DDCTNS312 = NULL))

            Technical Business Rule Format:
            ((^DDCTNS310 <> NULL OR ^DDCTNS311 <> NULL OR ^DDCTNS312 <> NULL) AND (^DDCTNS310 = NULL OR ^DDCTNS311 = NULL OR ^DDCTNS312 = NULL))
    
            Data Elements:
    
            ^DDCTNS310 = RP:IncomeTax.Deduction.LowValuePoolFinancialInvestments.Amount
    
            ^DDCTNS311 = RP:IncomeTax.Deduction.LowValuePoolRentalProperties.Amount
    
            ^DDCTNS312 = RP:IncomeTax.Deduction.LowValuePoolOther.Amount
            */
            assertion = ((report.DDCTNS310 != null || report.DDCTNS311 != null || report.DDCTNS312 != null) && (report.DDCTNS310 == null || report.DDCTNS311 == null || report.DDCTNS312 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000211",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"If any low value pool amount is provided, all must be provided",
                    LongDescription = @"If any of Low value pool deductions relating to financial investments, Low value pool deductions relating to rental properties or Other low value pool deductions are provided, all three must be provided",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.LowValuePoolFinancialInvestments.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000211" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "DDCTNS310", Value = GetValueOrEmpty(report.DDCTNS310) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "DDCTNS311", Value = GetValueOrEmpty(report.DDCTNS311) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "DDCTNS312", Value = GetValueOrEmpty(report.DDCTNS312) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000211

            #region VR.ATO.DDCTNS.000212

            /*  VR.ATO.DDCTNS.000212
            Invalid context. The number of Reporting party contexts must equal 1.
    
            Legacy Rule Format:
            (Count(RP) <> 1)

            Technical Business Rule Format:
            (Count(^RP:Context) <> 1)
    
            Data Elements:
            */
            assertion = (Count(report.RPCount) != 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430296",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting party contexts must equal 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000212" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000212

            #region VR.ATO.DDCTNS.000213

            /*  VR.ATO.DDCTNS.000213
            The number of Other Work Related Expenses contexts must not exceed 50.
    
            Legacy Rule Format:
            (Count(RP.{OTHWRSeqNum}) > 50)

            Technical Business Rule Format:
            (Count(^RP.{OTHWRSeqNum}:Context) > 50)
    
            Data Elements:
            */
            assertion = (Count(report.RPOTHWRSeqNumContextCollection) > 50);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000213",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Other Work Related Expenses contexts must not exceed 50.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOTHWRSeqNumContextCollection == null ? 0 : (report.RPOTHWRSeqNumContextCollection.Count() == 0 ? 0 : report.RPOTHWRSeqNumContextCollection.Last().LastOccurrenceIndex)),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000213" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000213

            #region VR.ATO.DDCTNS.000214

            /*  VR.ATO.DDCTNS.000214
            The Sequence Container within the Other Work Related Expenses Sequence Dimension context instance must not be duplicated
    
            Legacy Rule Format:
            (HasDuplicateValues(^DDCTNS313))

            Technical Business Rule Format:
            (HasDuplicateValues(^DDCTNS313))
    
            Data Elements:
    
            ^DDCTNS313 = RP.{OTHWRSeqNum}:Report.ItemOrder.Number
            */
            assertion = HasDuplicateValues(report.RPOTHWRSeqNumContextCollection == null ? null : report.RPOTHWRSeqNumContextCollection.Select(f => f.DDCTNS313).Cast<object>().ToArray());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000214",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Sequence Container must not be duplicated",
                    LongDescription = @"The Sequence Container within the Other Work Related Expenses Sequence Dimension context instance must not be duplicated",
                    Location = "/xbrli:xbrl/xbrli:context[@id='" + report.RPOTHWRSeqNumContextCollection[DuplicateValueIndex(report.RPOTHWRSeqNumContextCollection.Select(f => f.DDCTNS313))].Id + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000214" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000214

            #region VR.ATO.DDCTNS.000215

            /*  VR.ATO.DDCTNS.000215
            Where a Reporting Party Other work related expenses instance is present, Other work related expenses amount must be provided
    
            Legacy Rule Format:
            (AnyOccurrence(^DDCTNS315, ^DDCTNS315 = NULL))

            Technical Business Rule Format:
            (AnyOccurrence(^DDCTNS315, ^DDCTNS315 = NULL))
    
            Data Elements:
    
            ^DDCTNS315 = RP.{OTHWRSeqNum}:IncomeTax.Deduction.WorkRelatedOther.Amount
            */
            assertion = (report.RPOTHWRSeqNumContextCollection == null ? false : report.RPOTHWRSeqNumContextCollection.Any(DDCTNS315Repeat => DDCTNS315Repeat.DDCTNS315 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000215",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Other work related expenses amount must be provided",
                    LongDescription = @"Where a Reporting Party Other work related expenses instance is present, Other work related expenses amount must be provided",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.WorkRelatedOther.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000215" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "DDCTNS315", Value = "DDCTNS315" });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000215

            #region VR.ATO.DDCTNS.000216

            /*  VR.ATO.DDCTNS.000216
            The number of Reporting Party Interest Deductions contexts must not exceed 20.
    
            Legacy Rule Format:
            (Count(RP.{IntDedSeqNum}) > 20)

            Technical Business Rule Format:
            (Count(^RP.{IntDedSeqNum}:Context) > 20)
    
            Data Elements:
            */
            assertion = (Count(report.RPIntDedSeqNumContextCollection) > 20);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000216",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Interest Deductions contexts must not exceed 20.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPIntDedSeqNumContextCollection == null ? 0 : (report.RPIntDedSeqNumContextCollection.Count() == 0 ? 0 : report.RPIntDedSeqNumContextCollection.Last().LastOccurrenceIndex)),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000216" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000216

            #region VR.ATO.DDCTNS.000217

            /*  VR.ATO.DDCTNS.000217
            The Sequence Container within the Interest Deductions Sequence Dimension context instance must not be duplicated
    
            Legacy Rule Format:
            (HasDuplicateValues(^DDCTNS316))

            Technical Business Rule Format:
            (HasDuplicateValues(^DDCTNS316))
    
            Data Elements:
    
            ^DDCTNS316 = RP.{IntDedSeqNum}:Report.ItemOrder.Number
            */
            assertion = HasDuplicateValues(report.RPIntDedSeqNumContextCollection == null ? null : report.RPIntDedSeqNumContextCollection.Select(f => f.DDCTNS316).Cast<object>().ToArray());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000217",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Sequence Container must not be duplicated",
                    LongDescription = @"The Sequence Container within the Interest Deductions Sequence Dimension context instance must not be duplicated",
                    Location = "/xbrli:xbrl/xbrli:context[@id='" + report.RPIntDedSeqNumContextCollection[DuplicateValueIndex(report.RPIntDedSeqNumContextCollection.Select(f => f.DDCTNS316))].Id + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000217" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000217

            #region VR.ATO.DDCTNS.000218

            /*  VR.ATO.DDCTNS.000218
            Where a Reporting Party's Interest Deductions instance is present, Interest deduction amount must be provided
    
            Legacy Rule Format:
            (AnyOccurrence(^DDCTNS318, ^DDCTNS318 = NULL))

            Technical Business Rule Format:
            (AnyOccurrence(^DDCTNS318, ^DDCTNS318 = NULL))
    
            Data Elements:
    
            ^DDCTNS318 = RP.{IntDedSeqNum}:IncomeTax.Deduction.Interest.Amount
            */
            assertion = (report.RPIntDedSeqNumContextCollection == null ? false : report.RPIntDedSeqNumContextCollection.Any(DDCTNS318Repeat => DDCTNS318Repeat.DDCTNS318 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000218",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Interest deduction amount must be provided",
                    LongDescription = @"Where a Reporting Party Interest deductions instance is present, Interest deduction amount must be provided",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.Interest.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000218" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "DDCTNS318", Value = "DDCTNS318" });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000218

            #region VR.ATO.DDCTNS.000304

            /*  VR.ATO.DDCTNS.000304
            Superannuation fund Australian Business Number (ABN) has failed the ABN algorithm check.
    
            Legacy Rule Format:
            (AnyOccurrence(^DDCTNS405,(FailsABNAlgorithm(^DDCTNS405))))

            Technical Business Rule Format:
            (AnyOccurrence(^DDCTNS405,(FailsABNAlgorithm(^DDCTNS405))))
    
            Data Elements:
    
            ^DDCTNS405 = RP.{SCSeqNum}:Identifiers.AustralianBusinessNumber.Identifier
            */
            assertion = (report.RPSCSeqNumContextCollection == null ? false : report.RPSCSeqNumContextCollection.Any(DDCTNS405Repeat => FailsABNAlgorithm(DDCTNS405Repeat.DDCTNS405)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000304", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid ABN",
                    LongDescription = @"The Superannuation fund Australian Business Number (ABN) has failed the ABN algorithm check",
                    Location = "/xbrli:xbrl/tns:Identifiers.AustralianBusinessNumber.Identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000304"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "DDCTNS405", Value = "Count(DDCTNS405)" });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000304

            #region VR.ATO.DDCTNS.000305

            /*  VR.ATO.DDCTNS.000305
            The Superannuation fund Tax File Number has failed the algorithm check.
    
            Legacy Rule Format:
            (AnyOccurrence(^DDCTNS406,(FailsTFNAlgorithm(^DDCTNS406))))

            Technical Business Rule Format:
            (AnyOccurrence(^DDCTNS406,(FailsTFNAlgorithm(^DDCTNS406))))
    
            Data Elements:
    
            ^DDCTNS406 = RP.{SCSeqNum}:Identifiers.TaxFileNumber.Identifier
            */
            assertion = (report.RPSCSeqNumContextCollection == null ? false : report.RPSCSeqNumContextCollection.Any(DDCTNS406Repeat => FailsTFNAlgorithm(DDCTNS406Repeat.DDCTNS406)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000305", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid TFN",
                    LongDescription = @"The Superannuation fund Tax File Number (TFN) has failed the algorithm check",
                    Location = "/xbrli:xbrl/tns:Identifiers.TaxFileNumber.Identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000305"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "DDCTNS406", Value = "Count(DDCTNS406)" });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000305

            #region VR.ATO.DDCTNS.000306

            /*  VR.ATO.DDCTNS.000306
            The number of Reporting Party Personal Superannuation Contributions contexts must not exceed 25.
    
            Legacy Rule Format:
            IF COUNT(RP.{SCSeqNum}) > 25
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 25
    
            Data Elements:
                
            ^Context = RP:RP.{SCSeqNum}
            */
            assertion = (Count(report.RPSCSeqNumContextCollection) > 25);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000306", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting Party Personal Superannuation Contributions contexts must not exceed 25.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPSCSeqNumContextCollection == null ? 0 : (report.RPSCSeqNumContextCollection.Count() == 0 ? 0 : report.RPSCSeqNumContextCollection.Last().LastOccurrenceIndex)),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000306"} },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000306

            #region VR.ATO.DDCTNS.000307

            /*  VR.ATO.DDCTNS.000307
The Sequence Container within the Superannuation Contributions Sequence Dimension context instance must not be duplicated.
    
            Legacy Rule Format:
            (HasDuplicateValues(^DDCTNS400))

            Technical Business Rule Format:
            (HasDuplicateValues(^DDCTNS400))
    
            Data Elements:
    
            ^DDCTNS400 = RP.{SCSeqNum}:Report.ItemOrder.Number
            */
            assertion = HasDuplicateValues(report.RPSCSeqNumContextCollection == null ? null : report.RPSCSeqNumContextCollection.Select(f => f.DDCTNS400).Cast<object>().ToArray());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000307", Severity = ProcessMessageSeverity.Error,
                    Description = @"Sequence Container must not be duplicated",
                    LongDescription = @"The Sequence Container within the Superannuation Contributions Sequence Dimension context instance must not be duplicated",
                    Location = "/xbrli:xbrl/xbrli:context[@id='" + report.RPSCSeqNumContextCollection[DuplicateValueIndex(report.RPSCSeqNumContextCollection.Select(f => f.DDCTNS400))].Id + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000307"} },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000307

            #region VR.ATO.DDCTNS.000308

            /*  VR.ATO.DDCTNS.000308
            The number of Reporting Party Work Related Travel Expenses contexts must not exceed 20.
    
            Legacy Rule Format:
            IF COUNT(RP.{TrvlseqNum}) > 20
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 20
    
            Data Elements:
                
            ^Context = RP:RP.{TrvlseqNum}
            */
            assertion = (Count(report.RPTrvlSeqNumContextCollection) > 20);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000308", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting Party Work Related Travel expenses contexts must not exceed 20.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPTrvlSeqNumContextCollection == null ? 0 : (report.RPTrvlSeqNumContextCollection.Count() == 0 ? 0 : report.RPTrvlSeqNumContextCollection.Last().LastOccurrenceIndex)),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000308"} },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000308

            #region VR.ATO.DDCTNS.000309

            /*  VR.ATO.DDCTNS.000309
            The Sequence Container within the World Related Travel Expenses Sequence Dimension context instance must not be duplicated.
    
            Legacy Rule Format:
            (HasDuplicateValues(^DDCTNS408))

            Technical Business Rule Format:
            (HasDuplicateValues(^DDCTNS408))
    
            Data Elements:
    
            ^DDCTNS408 = RP.{TrvlSeqNum}:Report.ItemOrder.Number
            */
            assertion = HasDuplicateValues(report.RPTrvlSeqNumContextCollection == null ? null : report.RPTrvlSeqNumContextCollection.Select(f => f.DDCTNS408).Cast<object>().ToArray());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000309", Severity = ProcessMessageSeverity.Error,
                    Description = @"Sequence Container must not be duplicated",
                    LongDescription = @"The Sequence Container within the Work Related Travel expenses Sequence Dimension context instance must not be duplicated",
                    Location = "/xbrli:xbrl/xbrli:context[@id='" + report.RPTrvlSeqNumContextCollection[DuplicateValueIndex(report.RPTrvlSeqNumContextCollection.Select(f => f.DDCTNS408))].Id + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000309" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000309

            #region VR.ATO.DDCTNS.000310

            /*  VR.ATO.DDCTNS.000310
            Where a Reporting Party Personal Superannuation Contributions instance is present, Superannuation deduction claimed amount must be provided
    
            Legacy Rule Format:
            (AnyOccurrence(^DDCTNS407, ^DDCTNS407 = NULL))

            Technical Business Rule Format:
            (AnyOccurrence(^DDCTNS407, ^DDCTNS407 = NULL))
    
            Data Elements:
    
            ^DDCTNS407 = RP.{SCSeqNum}:IncomeTax.Deduction.SuperannuationContribution.Amount
            */
            assertion = (report.RPSCSeqNumContextCollection == null ? false : report.RPSCSeqNumContextCollection.Any(DDCTNS407Repeat => DDCTNS407Repeat.DDCTNS407 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000310",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Superannuation deduction claimed must be provided",
                    LongDescription = @"Where a Reporting Party Personal Superannuation Contributions instance is present, Superannuation deduction claimed amount must be provided",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.SuperannuationContribution.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000310" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "DDCTNS407", Value = "DDCTNS407" });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000310

            #region VR.ATO.DDCTNS.000313

            /*  VR.ATO.DDCTNS.000313
            Where a Reporting Party Travel Expenses instance is present, a Work related travel amount must be provided

            Legacy Rule Format:
            (AnyOccurrence(^DDCTNS410, ^DDCTNS410 = NULL))

            Technical Business Rule Format:
            (AnyOccurrence(^DDCTNS410, ^DDCTNS410 = NULL))

            Data Elements:

            ^DDCTNS410 = RP.{TrvlseqNum}:IncomeTax.Deduction.Travel.Amount
            */
            assertion = (report.RPTrvlSeqNumContextCollection == null ? false : report.RPTrvlSeqNumContextCollection.Any(DDCTNS410Repeat => DDCTNS410Repeat.DDCTNS410 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000313",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Work related travel amount must be provided",
                    LongDescription = @"Where a Reporting Party Travel expenses instance is present, a Work related travel amount must be provided",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.Travel.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000313" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "DDCTNS410", Value = "DDCTNS410" });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000313

            #region VR.ATO.DDCTNS.000314

            /*  VR.ATO.DDCTNS.000314
            The number of Reporting Party Gifts or Donation contexts must not exceed 20.
    
            Legacy Rule Format:
            (Count(RP.{GiftDonSeqNum}) > 20)

            Technical Business Rule Format:
            (Count(^RP.{GiftDonSeqNum}:Context) > 20)
    
            Data Elements:
            */
            assertion = (Count(report.RPGiftDonSeqNumContextCollection) > 20);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000314",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Gifts or Donations contexts must not exceed 20.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPGiftDonSeqNumContextCollection == null ? 0 : (report.RPGiftDonSeqNumContextCollection.Count() == 0 ? 0 : report.RPGiftDonSeqNumContextCollection.Last().LastOccurrenceIndex)),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000314" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000314

            #region VR.ATO.DDCTNS.000315

            /*  VR.ATO.DDCTNS.000315
            The Sequence Container within the Gifts or donations Sequence Dimension context instance must not be duplicated
    
            Legacy Rule Format:
            (HasDuplicateValues(^DDCTNS411))

            Technical Business Rule Format:
            (HasDuplicateValues(^DDCTNS411))
    
            Data Elements:
    
            ^DDCTNS411 = RP.{GiftDonSeqNum}:Report.ItemOrder.Number
            */
            assertion = HasDuplicateValues(report.RPGiftDonSeqNumContextCollection == null ? null : report.RPGiftDonSeqNumContextCollection.Select(f => f.DDCTNS411).Cast<object>().ToArray());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000315",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Sequence Container must not be duplicated",
                    LongDescription = @"The Sequence Container within the Gifts or donations Sequence Dimension context instance must not be duplicated",
                    Location = "/xbrli:xbrl/xbrli:context[@id='" + report.RPGiftDonSeqNumContextCollection[DuplicateValueIndex(report.RPGiftDonSeqNumContextCollection.Select(f => f.DDCTNS411))].Id + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000315" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000315

            #region VR.ATO.DDCTNS.000316

            /*  VR.ATO.DDCTNS.000316
            Where a Reporting Party Gifts or Donations instance is present, a Gifts or donations amount must be provided
    
            Legacy Rule Format:
            (AnyOccurrence(^DDCTNS413, ^DDCTNS413 = NULL))

            Technical Business Rule Format:
            (AnyOccurrence(^DDCTNS413, ^DDCTNS413 = NULL))
    
            Data Elements:
    
            ^DDCTNS413 = RP.{GiftDonSeqNum}:IncomeTax.Deduction.GiftDonation.Amount
            */
            assertion = (report.RPGiftDonSeqNumContextCollection == null ? false : report.RPGiftDonSeqNumContextCollection.Any(DDCTNS413Repeat => DDCTNS413Repeat.DDCTNS413 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000316",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Gifts or donations amount must be provided",
                    LongDescription = @"Where a Reporting Party Gifts or Donations instance is present, a Gifts or donations amount must be provided",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.GiftDonation.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000316" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "DDCTNS413", Value = "DDCTNS413" });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000316

            #region VR.ATO.DDCTNS.000317

            /*  VR.ATO.DDCTNS.000317
            The number of Reporting Party Work Related Clothing expenses contexts must not exceed 20.
    
            Legacy Rule Format:
            (Count(RP.{ClthngSeqNum}) > 20)

            Technical Business Rule Format:
            (Count(^RP.{ClthngSeqNum}:Context) > 20)
    
            Data Elements:
            */
            assertion = (Count(report.RPClthngSeqNumContextCollection) > 20);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000317",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting Party Work Related Clothing expenses contexts must not exceed 20.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPClthngSeqNumContextCollection == null ? 0 : (report.RPClthngSeqNumContextCollection.Count() == 0 ? 0 : report.RPClthngSeqNumContextCollection.Last().LastOccurrenceIndex)),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000317" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000317

            #region VR.ATO.DDCTNS.000318

            /*  VR.ATO.DDCTNS.000318
            The Sequence Container within the Work Related Clothing expenses Sequence Dimension context instance must not be duplicated
    
            Legacy Rule Format:
            (HasDuplicateValues(^DDCTNS414))

            Technical Business Rule Format:
            (HasDuplicateValues(^DDCTNS414))
    
            Data Elements:
    
            ^DDCTNS414 = RP.{ClthngSeqNum}:Report.ItemOrder.Number
            */
            assertion = HasDuplicateValues(report.RPClthngSeqNumContextCollection == null ? null : report.RPClthngSeqNumContextCollection.Select(f => f.DDCTNS414).Cast<object>().ToArray());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000318",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Sequence Container must not be duplicated",
                    LongDescription = @"The Sequence Container within the Work Related Clothing expenses Sequence Dimension context instance must not be duplicated",
                    Location = "/xbrli:xbrl/xbrli:context[@id='" + report.RPClthngSeqNumContextCollection[DuplicateValueIndex(report.RPClthngSeqNumContextCollection.Select(f => f.DDCTNS414))].Id + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000318" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000318

            #region VR.ATO.DDCTNS.000320

            /*  VR.ATO.DDCTNS.000320
            Where a Reporting Party Clothing expenses instance is present, a Work related clothing expenses amount must be provided
    
            Legacy Rule Format:
            (AnyOccurrence(^DDCTNS416, ^DDCTNS416 = NULL))

            Technical Business Rule Format:
            (AnyOccurrence(^DDCTNS416, ^DDCTNS416 = NULL))
    
            Data Elements:
    
            ^DDCTNS416 = RP.{ClthngSeqNum}:IncomeTax.Deduction.Clothing.Amount
            */
            assertion = (report.RPClthngSeqNumContextCollection == null ? false : report.RPClthngSeqNumContextCollection.Any(DDCTNS416Repeat => DDCTNS416Repeat.DDCTNS416 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000320",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Work related clothing expenses amount must be provided",
                    LongDescription = @"Where a Reporting Party Clothing expenses instance is present, a Work related clothing expenses amount must be provided",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.Clothing.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000320" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "DDCTNS416", Value = "DDCTNS416" });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.DDCTNS.000320

            #region Repeating RPCarSeqNumContext
            if (report.RPCarSeqNumContextCollection != null)
            {
                int itemIndexContext2 = 0;
                foreach (DDCTNS2019.RPCarSeqNumContext rpCarSeqNumContext in report.RPCarSeqNumContextCollection)
                {
                    itemIndexContext2++;

                    #region VR.ATO.DDCTNS.000003

                    /*  VR.ATO.DDCTNS.000003
                    Where a Reporting Party Car instance is present, Car expenses calculation method must be provided
    
                    Legacy Rule Format:
                    (AnyOccurrence(^DDCTNS102, ^DDCTNS102 = NULL))
        
                    Technical Business Rule Format:
                    (AnyOccurrence(^DDCTNS102, ^DDCTNS102 = NULL))
            
                    Data Elements:
            
                    ^DDCTNS102 = RP.{CarSeqNum}:IncomeTax.Deduction.CarCalculationMethod.Code
                    */
                    assertion = (rpCarSeqNumContext.DDCTNS102 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.DDCTNS.000003",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Where a Reporting Party Car instance is present, Car expenses calculation method must be provided",
                            Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.CarCalculationMethod.Code[@contextRef='" + rpCarSeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000003" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS102", Value = rpCarSeqNumContext.DDCTNS102 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.DDCTNS.000003

                    #region VR.ATO.DDCTNS.000004

                    /*  VR.ATO.DDCTNS.000004
                    When the Car expenses calculation method is "Cents per kilometre", logbook method details must not be provided
    
                    Legacy Rule Format:
                    (^DDCTNS102 = "Cents per kilometre" AND (^DDCTNS104 <> NULL OR ^DDCTNS105 <> NULL OR ^DDCTNS106 <> NULL))
        
                    Technical Business Rule Format:
                    (^DDCTNS102 = 'Cents per kilometre' AND (^DDCTNS104 <> NULL OR ^DDCTNS105 <> NULL OR ^DDCTNS106 <> NULL))
            
                    Data Elements:
            
                    ^DDCTNS102 = RP.{CarSeqNum}:IncomeTax.Deduction.CarCalculationMethod.Code
            
                    ^DDCTNS104 = RP.{CarSeqNum}:IncomeTax.Deduction.CarLogBookMethodBusinessUseRecorded.Percent
            
                    ^DDCTNS105 = RP.{CarSeqNum}:IncomeTax.Deduction.CarLogBookMethod.Amount
            
                    ^DDCTNS106 = RP.{CarSeqNum}:IncomeTax.Deduction.CarLogBookMethodDepreciation.Amount
                    */
                    assertion = (rpCarSeqNumContext.DDCTNS102 == @"Cents per kilometre" && (rpCarSeqNumContext.DDCTNS104 != null || rpCarSeqNumContext.DDCTNS105 != null || rpCarSeqNumContext.DDCTNS106 != null));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.DDCTNS.000004",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Incorrect Car expenses calculation method details provided",
                            LongDescription = @"When the Car expenses calculation method is ""Cents per kilometre"", logbook method details must not be provided",
                            Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.CarCalculationMethod.Code[@contextRef='" + rpCarSeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000004" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS102", Value = rpCarSeqNumContext.DDCTNS102 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS104", Value = rpCarSeqNumContext.DDCTNS104.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS105", Value = GetValueOrEmpty(rpCarSeqNumContext.DDCTNS105) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS106", Value = GetValueOrEmpty(rpCarSeqNumContext.DDCTNS106) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.DDCTNS.000004

                    #region VR.ATO.DDCTNS.000005

                    /*  VR.ATO.DDCTNS.000005
                    When the Car expenses calculation method is "Logbook", Car expenses business kilometres travelled must not be provided
    
                    Legacy Rule Format:
                    (^DDCTNS102 = "Logbook" AND ^DDCTNS103 <> NULL)
        
                    Technical Business Rule Format:
                    ^DDCTNS102 = 'Logbook' AND ^DDCTNS103 <> NULL
            
                    Data Elements:
            
                    ^DDCTNS102 = RP.{CarSeqNum}:IncomeTax.Deduction.CarCalculationMethod.Code
            
                    ^DDCTNS103 = RP.{CarSeqNum}:IncomeTax.Deduction.CarBusinessKilometres.Number
                    */
                    assertion = (rpCarSeqNumContext.DDCTNS102 == @"Logbook" && rpCarSeqNumContext.DDCTNS103 != null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.DDCTNS.000005",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Incorrect Car expenses calculation method details provided",
                            LongDescription = @"When the Car expenses calculation method is ""Logbook"", Car expenses business kilometres travelled must not be provided",
                            Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.CarCalculationMethod.Code[@contextRef='" + rpCarSeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000005" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS102", Value = rpCarSeqNumContext.DDCTNS102 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS103", Value = rpCarSeqNumContext.DDCTNS103.GetValueOrDefault().ToString() });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.DDCTNS.000005

                    #region VR.ATO.DDCTNS.000006

                    /*  VR.ATO.DDCTNS.000006
                    When the Car expenses calculation method is "Cents per kilometre", Car expenses business kilometres travelled must be provided
    
                    Legacy Rule Format:
                    (^DDCTNS102 = "Cents per kilometre" AND ^DDCTNS103 = NULL)
        
                    Technical Business Rule Format:
                    ^DDCTNS102 = 'Cents per kilometre' AND ^DDCTNS103 = NULL
            
                    Data Elements:
            
                    ^DDCTNS103 = RP.{CarSeqNum}:IncomeTax.Deduction.CarBusinessKilometres.Number
            
                    ^DDCTNS102 = RP.{CarSeqNum}:IncomeTax.Deduction.CarCalculationMethod.Code
                    */
                    assertion = (rpCarSeqNumContext.DDCTNS102 == @"Cents per kilometre" && rpCarSeqNumContext.DDCTNS103 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.DDCTNS.000006",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Car expenses business kilometres travelled must be provided",
                            LongDescription = @"When the Car expenses calculation method is ""Cents per kilometre"", Car expenses business kilometres travelled must be provided",
                            Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.CarBusinessKilometres.Number[@contextRef='" + rpCarSeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000006" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS102", Value = rpCarSeqNumContext.DDCTNS102 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS103", Value = rpCarSeqNumContext.DDCTNS103.GetValueOrDefault().ToString() });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.DDCTNS.000006

                    #region VR.ATO.DDCTNS.000008

                    /*  VR.ATO.DDCTNS.000008
                    When the Car expenses calculation method is "Logbook", both Car expenses percentage of business use and Car expenses amount must be provided
    
                    Legacy Rule Format:
                    (^DDCTNS102 = "Logbook") AND (^DDCTNS104 = NULL OR ^DDCTNS105 = NULL)
        
                    Technical Business Rule Format:
                    (^DDCTNS102 = 'Logbook') AND (^DDCTNS104 = NULL OR ^DDCTNS105 = NULL)
            
                    Data Elements:
            
                    ^DDCTNS104 = RP.{CarSeqNum}:IncomeTax.Deduction.CarLogBookMethodBusinessUseRecorded.Percent
            
                    ^DDCTNS102 = RP.{CarSeqNum}:IncomeTax.Deduction.CarCalculationMethod.Code
            
                    ^DDCTNS105 = RP.{CarSeqNum}:IncomeTax.Deduction.CarLogBookMethod.Amount
                    */
                    assertion = (rpCarSeqNumContext.DDCTNS102 == @"Logbook" && (rpCarSeqNumContext.DDCTNS104 == null || rpCarSeqNumContext.DDCTNS105 == null));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.DDCTNS.000008",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Car expenses logbook details must be provided",
                            LongDescription = @"When the Car expenses calculation method is ""Logbook"", both Car expenses percentage of business use and Car expenses amount must be provided",
                            Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.CarLogBookMethodBusinessUseRecorded.Percent[@contextRef='" + rpCarSeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000008" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS102", Value = rpCarSeqNumContext.DDCTNS102 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS104", Value = rpCarSeqNumContext.DDCTNS104.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS105", Value = GetValueOrEmpty(rpCarSeqNumContext.DDCTNS105) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.DDCTNS.000008
                }
            }

            #endregion // Foreach loop

            #region Repeating RPEduSeqNumContext
            if (report.RPEduSeqNumContextCollection != null)
            {
                int itemIndexContext2 = 0;
                foreach (DDCTNS2019.RPEduSeqNumContext rpEduSeqNumContext in report.RPEduSeqNumContextCollection)
                {
                    itemIndexContext2++;

                    #region VR.ATO.DDCTNS.000010

                    /*  VR.ATO.DDCTNS.000010
                    Self education expense category must be "Car expenses" when Self education car expenses fields are provided
    
                    Legacy Rule Format:
                    (^DDCTNS202 <> "Car expenses") AND (^DDCTNS120 <> NULL OR ^DDCTNS121 <> NULL OR ^DDCTNS122 <> NULL OR ^DDCTNS123 <> NULL OR ^DDCTNS124 <> NULL OR ^DDCTNS125 <> NULL)
        
                    Technical Business Rule Format:
                    (^DDCTNS202 <> 'Car expenses') AND (^DDCTNS120 <> NULL OR ^DDCTNS121 <> NULL OR ^DDCTNS122 <> NULL OR ^DDCTNS123 <> NULL OR ^DDCTNS124 <> NULL OR ^DDCTNS125 <> NULL)
            
                    Data Elements:
            
                    ^DDCTNS202 = RP.{EduSeqNum}:IncomeTax.Deduction.SelfEducation.Code
            
                    ^DDCTNS120 = RP.{EduSeqNum}:Assets.CarMakeModel.Description
            
                    ^DDCTNS121 = RP.{EduSeqNum}:IncomeTax.Deduction.CarCalculationMethod.Code
            
                    ^DDCTNS122 = RP.{EduSeqNum}:IncomeTax.Deduction.CarBusinessKilometres.Number
            
                    ^DDCTNS123 = RP.{EduSeqNum}:IncomeTax.Deduction.CarLogBookMethodBusinessUseRecorded.Percent
            
                    ^DDCTNS124 = RP.{EduSeqNum}:IncomeTax.Deduction.CarLogBookMethod.Amount
            
                    ^DDCTNS125 = RP.{EduSeqNum}:IncomeTax.Deduction.CarLogBookMethodDepreciation.Amount
                    */
                    assertion = (rpEduSeqNumContext.DDCTNS202 != @"Car expenses" && (rpEduSeqNumContext.DDCTNS120 != null || rpEduSeqNumContext.DDCTNS121 != null || rpEduSeqNumContext.DDCTNS122 != null || rpEduSeqNumContext.DDCTNS123 != null || rpEduSeqNumContext.DDCTNS124 != null || rpEduSeqNumContext.DDCTNS125 != null));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.DDCTNS.000010",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Self education expense category must be ""Car expenses"" when Self education car expenses fields are provided",
                            Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.SelfEducation.Code[@contextRef='" + rpEduSeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000010" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS202", Value = rpEduSeqNumContext.DDCTNS202 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS120", Value = GetValueOrEmpty(rpEduSeqNumContext.DDCTNS120) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS121", Value = GetValueOrEmpty(rpEduSeqNumContext.DDCTNS121) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS122", Value = rpEduSeqNumContext.DDCTNS122.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS123", Value = rpEduSeqNumContext.DDCTNS123.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS124", Value = GetValueOrEmpty(rpEduSeqNumContext.DDCTNS124) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS125", Value = GetValueOrEmpty(rpEduSeqNumContext.DDCTNS125) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.DDCTNS.000010

                    #region VR.ATO.DDCTNS.000011

                    /*  VR.ATO.DDCTNS.000011
                    Where Self education expense category is "Car expenses", Self education car expenses calculation method must be provided
    
                    Legacy Rule Format:
                    (^DDCTNS202 = "Car expenses") AND (^DDCTNS121 = NULL)
        
                    Technical Business Rule Format:
                    (^DDCTNS202 = 'Car expenses') AND (^DDCTNS121 = NULL)
            
                    Data Elements:
            
                    ^DDCTNS121 = RP.{EduSeqNum}:IncomeTax.Deduction.CarCalculationMethod.Code
            
                    ^DDCTNS202 = RP.{EduSeqNum}:IncomeTax.Deduction.SelfEducation.Code
                    */
                    assertion = (rpEduSeqNumContext.DDCTNS202 == @"Car expenses" && rpEduSeqNumContext.DDCTNS121 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.DDCTNS.000011",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Self education car expenses calculation method must be provided",
                            LongDescription = @"Where Self education expense category is ""Car expenses"", Self education car expenses calculation method must be provided",
                            Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.CarCalculationMethod.Code[@contextRef='" + rpEduSeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000011" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS202", Value = rpEduSeqNumContext.DDCTNS202 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS121", Value = GetValueOrEmpty(rpEduSeqNumContext.DDCTNS121) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.DDCTNS.000011

                    #region VR.ATO.DDCTNS.000012

                    /*  VR.ATO.DDCTNS.000012
                    When the Self education car expenses calculation method is "Cents per kilometre", logbook method details must not be provided
    
                    Legacy Rule Format:
                    (^DDCTNS121 = "Cents per kilometre") AND (^DDCTNS123 <> NULL OR ^DDCTNS124 <> NULL OR ^DDCTNS125 <> NULL)
        
                    Technical Business Rule Format:
                    (^DDCTNS121 = 'Cents per kilometre') AND (^DDCTNS123 <> NULL OR ^DDCTNS124 <> NULL OR ^DDCTNS125 <> NULL)
            
                    Data Elements:
            
                    ^DDCTNS121 = RP.{EduSeqNum}:IncomeTax.Deduction.CarCalculationMethod.Code
            
                    ^DDCTNS123 = RP.{EduSeqNum}:IncomeTax.Deduction.CarLogBookMethodBusinessUseRecorded.Percent
            
                    ^DDCTNS124 = RP.{EduSeqNum}:IncomeTax.Deduction.CarLogBookMethod.Amount
            
                    ^DDCTNS125 = RP.{EduSeqNum}:IncomeTax.Deduction.CarLogBookMethodDepreciation.Amount
                    */
                    assertion = (rpEduSeqNumContext.DDCTNS121 == @"Cents per kilometre" && (rpEduSeqNumContext.DDCTNS123 != null || rpEduSeqNumContext.DDCTNS124 != null || rpEduSeqNumContext.DDCTNS125 != null));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.DDCTNS.000012",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Incorrect Self education car expenses calculation method details provided",
                            LongDescription = @"When the Self education car expenses calculation method is ""Cents per kilometre"", logbook method details must not be provided",
                            Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.CarCalculationMethod.Code[@contextRef='" + rpEduSeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000012" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS121", Value = GetValueOrEmpty(rpEduSeqNumContext.DDCTNS121) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS123", Value = rpEduSeqNumContext.DDCTNS123.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS124", Value = GetValueOrEmpty(rpEduSeqNumContext.DDCTNS124) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS125", Value = GetValueOrEmpty(rpEduSeqNumContext.DDCTNS125) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.DDCTNS.000012

                    #region VR.ATO.DDCTNS.000013

                    /*  VR.ATO.DDCTNS.000013
                    When the Self education expenses calculation method is "Logbook", Self education car expenses business kilometres travelled must not be provided
    
                    Legacy Rule Format:
                    (^DDCTNS121 = "Logbook") AND (^DDCTNS122 <> NULL)
        
                    Technical Business Rule Format:
                    (^DDCTNS121 = 'Logbook') AND (^DDCTNS122 <> NULL)
            
                    Data Elements:
            
                    ^DDCTNS121 = RP.{EduSeqNum}:IncomeTax.Deduction.CarCalculationMethod.Code
            
                    ^DDCTNS122 = RP.{EduSeqNum}:IncomeTax.Deduction.CarBusinessKilometres.Number
                    */
                    assertion = (rpEduSeqNumContext.DDCTNS121 == @"Logbook" && rpEduSeqNumContext.DDCTNS122 != null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.DDCTNS.000013",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Incorrect Self education car expenses calculation method details provided",
                            LongDescription = @"When the Self education expenses calculation method is ""Logbook"", Self education car expenses business kilometres travelled must not be provided",
                            Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.CarCalculationMethod.Code[@contextRef='" + rpEduSeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000013" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS121", Value = GetValueOrEmpty(rpEduSeqNumContext.DDCTNS121) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS122", Value = rpEduSeqNumContext.DDCTNS122.GetValueOrDefault().ToString() });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.DDCTNS.000013

                    #region VR.ATO.DDCTNS.000014

                    /*  VR.ATO.DDCTNS.000014
                    When the Self education car expenses calculation method is "Cents per kilometre", Self education car expenses business kilometres travelled must be provided
    
                    Legacy Rule Format:
                    (^DDCTNS121 = "Cents per kilometre") AND (^DDCTNS122 = NULL)
        
                    Technical Business Rule Format:
                    (^DDCTNS121 = 'Cents per kilometre') AND (^DDCTNS122 = NULL)
            
                    Data Elements:
            
                    ^DDCTNS122 = RP.{EduSeqNum}:IncomeTax.Deduction.CarBusinessKilometres.Number
            
                    ^DDCTNS121 = RP.{EduSeqNum}:IncomeTax.Deduction.CarCalculationMethod.Code
                    */
                    assertion = (rpEduSeqNumContext.DDCTNS121 == @"Cents per kilometre" && rpEduSeqNumContext.DDCTNS122 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.DDCTNS.000014",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Self education car expenses business kilometres travelled must be provided",
                            LongDescription = @"When the Self education car expenses calculation method is ""Cents per kilometre"", Self education car expenses business kilometres travelled must be provided",
                            Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.CarBusinessKilometres.Number[@contextRef='" + rpEduSeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000014" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS121", Value = GetValueOrEmpty(rpEduSeqNumContext.DDCTNS121) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS122", Value = rpEduSeqNumContext.DDCTNS122.GetValueOrDefault().ToString() });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.DDCTNS.000014

                    #region VR.ATO.DDCTNS.000015

                    /*  VR.ATO.DDCTNS.000015
                    When the Self education car expenses calculation method is "Logbook", both Self education car expenses percentage of business use and Self education car expenses amount must be provided
    
                    Legacy Rule Format:
                    (^DDCTNS121 = "Logbook") AND (^DDCTNS123 = NULL OR ^DDCTNS124 = NULL)
        
                    Technical Business Rule Format:
                    (^DDCTNS121 = 'Logbook') AND (^DDCTNS123 = NULL OR ^DDCTNS124 = NULL)
            
                    Data Elements:
            
                    ^DDCTNS123 = RP.{EduSeqNum}:IncomeTax.Deduction.CarLogBookMethodBusinessUseRecorded.Percent
            
                    ^DDCTNS121 = RP.{EduSeqNum}:IncomeTax.Deduction.CarCalculationMethod.Code
            
                    ^DDCTNS124 = RP.{EduSeqNum}:IncomeTax.Deduction.CarLogBookMethod.Amount
                    */
                    assertion = (rpEduSeqNumContext.DDCTNS121 == @"Logbook" && (rpEduSeqNumContext.DDCTNS123 == null || rpEduSeqNumContext.DDCTNS124 == null));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.DDCTNS.000015",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Self education car expenses logbook method details must be provided",
                            LongDescription = @"When the Self education car expenses calculation method is ""Logbook"", both Self education car expenses percentage of business use and Self education car expenses amount must be provided",
                            Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.CarLogBookMethodBusinessUseRecorded.Percent[@contextRef='" + rpEduSeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000015" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS121", Value = GetValueOrEmpty(rpEduSeqNumContext.DDCTNS121) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS123", Value = rpEduSeqNumContext.DDCTNS123.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS124", Value = GetValueOrEmpty(rpEduSeqNumContext.DDCTNS124) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.DDCTNS.000015

                    #region VR.ATO.DDCTNS.000104

                    /*  VR.ATO.DDCTNS.000104
                    When the Self education expense/depreciation amount is present, the only valid Self education expense category codes are; "Other", "Decline in value", or "Repair expenses"
    
                    Legacy Rule Format:
                    (^DDCTNS202 = "Car expenses") AND (^DDCTNS204 <> NULL)
        
                    Technical Business Rule Format:
                    (^DDCTNS202 = 'Car expenses') AND (^DDCTNS204 <> NULL)
            
                    Data Elements:
            
                    ^DDCTNS202 = RP.{EduSeqNum}:IncomeTax.Deduction.SelfEducation.Code
            
                    ^DDCTNS204 = RP.{EduSeqNum}:IncomeTax.Deduction.Education.Amount
                    */
                    assertion = (rpEduSeqNumContext.DDCTNS202 == @"Car expenses" && rpEduSeqNumContext.DDCTNS204 != null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.DDCTNS.000342",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Self education expense category is incorrect",
                            LongDescription = @"When the Self education expense/depreciation amount is present, the only valid Self education expense category codes are; ""Other"", ""Decline in value"", or ""Repair expenses""",
                            Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.SelfEducation.Code[@contextRef='" + rpEduSeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000104" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS202", Value = rpEduSeqNumContext.DDCTNS202 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS204", Value = GetValueOrEmpty(rpEduSeqNumContext.DDCTNS204) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.DDCTNS.000104

                    #region VR.ATO.DDCTNS.000105

                    /*  VR.ATO.DDCTNS.000105
                    When the Self education expense description is present, the Self education expense/depreciation amount must be provided
    
                    Legacy Rule Format:
                    (^DDCTNS203 <> NULL) AND (^DDCTNS204 = NULL)
        
                    Technical Business Rule Format:
                    (^DDCTNS203 <> NULL) AND (^DDCTNS204 = NULL)
            
                    Data Elements:
            
                    ^DDCTNS203 = RP.{EduSeqNum}:IncomeTax.Deduction.EducationCategoryOther.Description
            
                    ^DDCTNS204 = RP.{EduSeqNum}:IncomeTax.Deduction.Education.Amount
                    */
                    assertion = (rpEduSeqNumContext.DDCTNS203 != null && rpEduSeqNumContext.DDCTNS204 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.DDCTNS.000105", Severity = ProcessMessageSeverity.Error,
                            Description = @"Self education expense/depreciation amount must be provided",
                            LongDescription = @"When the Self education expense description is present, the Self education expense/depreciation amount must be provided",
                            Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.EducationCategoryOther.Description[@contextRef='" + rpEduSeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000105"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS203", Value = GetValueOrEmpty(rpEduSeqNumContext.DDCTNS203) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS204", Value = GetValueOrEmpty(rpEduSeqNumContext.DDCTNS204) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.DDCTNS.000105

                    #region VR.ATO.DDCTNS.000106

                    /*  VR.ATO.DDCTNS.000106
                    The Self education expense/depreciation amount must be provided when the Self education expense category is "Other", "Decline in value", or "Repair expenses"
    
                    Legacy Rule Format:
                    (^DDCTNS204 = NULL) AND (^DDCTNS202 <> NULL) AND (^DDCTNS202 <> "Car expenses")
        
                    Technical Business Rule Format:
                    (^DDCTNS204 = NULL) AND (^DDCTNS202 <> NULL) AND (^DDCTNS202 <> "Car expenses")
            
                    Data Elements:
            
                    ^DDCTNS204 = RP.{EduSeqNum}:IncomeTax.Deduction.Education.Amount
            
                    ^DDCTNS202 = RP.{EduSeqNum}:IncomeTax.Deduction.SelfEducation.Code
                    */
                    assertion = (rpEduSeqNumContext.DDCTNS204 == null && rpEduSeqNumContext.DDCTNS202 != null && rpEduSeqNumContext.DDCTNS202 != @"Car expenses");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.DDCTNS.000343",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Self education expense/depreciation amount must be provided",
                            LongDescription = @"The Self education expense/depreciation amount must be provided when the Self education expense category is ""Other"", ""Decline in value"", or ""Repair expenses""",
                            Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.Education.Amount[@contextRef='" + rpEduSeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000106" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS204", Value = GetValueOrEmpty(rpEduSeqNumContext.DDCTNS204) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS202", Value = rpEduSeqNumContext.DDCTNS202 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.DDCTNS.000106
                }
            }

            #endregion // Foreach loop

            #region Repeating RPSCSeqNumContext
            if (report.RPSCSeqNumContextCollection != null)
            {
                int itemIndexContext2 = 0;
                foreach (DDCTNS2019.RPSCSeqNumContext rpscSeqNumContext in report.RPSCSeqNumContextCollection)
                {
                    itemIndexContext2++;

                    #region VR.ATO.DDCTNS.000300

                    /*  VR.ATO.DDCTNS.000300
                    To be eligible to claim a deduction for personal super contributions, an individual must provide their fund with a notice of intent and their fund must acknowledge that notice before they lodge their tax return.
    
                    Legacy Rule Format:
                    (^DDCTNS401 = NULL OR ^DDCTNS401 = FALSE) AND (^DDCTNS407 > 0)
        
                    Technical Business Rule Format:
                    (^DDCTNS401 = NULL OR ^DDCTNS401 = FALSE) AND (^DDCTNS407 > 0)
            
                    Data Elements:
            
                    ^DDCTNS401 = RP.{SCSeqNum}:SuperannuationContribution.NOIProvidedAcknowledgmentReceived.Indicator
            
                    ^DDCTNS407 = RP.{SCSeqNum}:IncomeTax.Deduction.SuperannuationContribution.Amount
                    */
                    assertion = ((rpscSeqNumContext.DDCTNS401 == null || rpscSeqNumContext.DDCTNS401 == false) && rpscSeqNumContext.DDCTNS407.GetValueOrDefault() > 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.DDCTNS.000300", Severity = ProcessMessageSeverity.Error,
                            Description = @"Answer to 'Did you provide your fund (including a retirement savings account) with a notice of intent to claim a deduction for personal superannuation contributions, and receive an acknowledgement from your fund?' must be true",
                            LongDescription = @"To be eligible to claim a deduction for personal super contributions, the response to 'Did you provide your fund (including a retirement savings account) with a notice of intent to claim a deduction for personal superannuation contributions, and receive an acknowledgement from your fund?' must be 'yes' (true)",
                            Location = "/xbrli:xbrl/tns:SuperannuationContribution.NOIProvidedAcknowledgmentReceived.Indicator[@contextRef='" + rpscSeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000300"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS401", Value = GetValueOrEmpty(rpscSeqNumContext.DDCTNS401) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS407", Value = GetValueOrEmpty(rpscSeqNumContext.DDCTNS407) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.DDCTNS.000300

                    #region VR.ATO.DDCTNS.000301

                    /*  VR.ATO.DDCTNS.000301
                    Where a Personal Superannuation Contributions (SC) instance has been provided, at least one of the SC details must be provided.
    
                    Legacy Rule Format:
                    ((^DDCTNS402 = NULLORBLANK) AND (^DDCTNS404 = NULLORBLANK) AND (^DDCTNS407 = NULL) AND (^DDCTNS405 = NULL) AND (^DDCTNS406 = NULL))
        
                    Technical Business Rule Format:
                    ((^DDCTNS402 = BLANK) AND (^DDCTNS404 = BLANK) AND (^DDCTNS407 = NULL) AND (^DDCTNS405 = NULL) AND (^DDCTNS406 = NULL))
            
                    Data Elements:
            
                    ^DDCTNS402 = RP.{SCSeqNum}:OrganisationNameDetails.OrganisationalName.Text
            
                    ^DDCTNS404 = RP.{SCSeqNum}:SuperannuationFundDetails.MemberAccount.Identifier
            
                    ^DDCTNS405 = RP.{SCSeqNum}:Identifiers.AustralianBusinessNumber.Identifier
            
                    ^DDCTNS406 = RP.{SCSeqNum}:Identifiers.TaxFileNumber.Identifier
            
                    ^DDCTNS407 = RP.{SCSeqNum}:IncomeTax.Deduction.SuperannuationContribution.Amount
                    */
                    assertion = (string.IsNullOrWhiteSpace(rpscSeqNumContext.DDCTNS402) == true && string.IsNullOrWhiteSpace(rpscSeqNumContext.DDCTNS404) == true && rpscSeqNumContext.DDCTNS407 == null && rpscSeqNumContext.DDCTNS405 == null && rpscSeqNumContext.DDCTNS406 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.DDCTNS.000301", Severity = ProcessMessageSeverity.Error,
                            Description = @"At least one of the Personal Superannuation Contributions details must be provided",
                            LongDescription = @"Where a Personal Superannuation Contributions (SC) instance has been provided, at least one of the SC details must be provided",
                            Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text[@contextRef='" + rpscSeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000301"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS402", Value = GetValueOrEmpty(rpscSeqNumContext.DDCTNS402) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS404", Value = GetValueOrEmpty(rpscSeqNumContext.DDCTNS404) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS407", Value = GetValueOrEmpty(rpscSeqNumContext.DDCTNS407) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS405", Value = GetValueOrEmpty(rpscSeqNumContext.DDCTNS405) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS406", Value = GetValueOrEmpty(rpscSeqNumContext.DDCTNS406) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.DDCTNS.000301

                    #region VR.ATO.DDCTNS.000303

                    /*  VR.ATO.DDCTNS.000303
                    An amount is present at Deduction claimed for Personal Superannuation Contributions. Either the Full superannuation fund name and Superannuation account number are not populated, or the Superannuation fund Australian Business Number (ABN) and Superannuation fund Tax File Number (TFN) are not populated.
    
                    Legacy Rule Format:
                    (^DDCTNS407 > 0 AND ((^DDCTNS402 = NULLORBLANK AND ^DDCTNS404 = NULLORBLANK) OR (^DDCTNS405 = NULL AND ^DDCTNS406 = NULL)))
        
                    Technical Business Rule Format:
                    (^DDCTNS407 > 0 AND ((^DDCTNS402 = BLANK AND ^DDCTNS404 = BLANK) OR (^DDCTNS405 = NULL AND ^DDCTNS406 = NULL)))
            
                    Data Elements:
            
                    ^DDCTNS407 = RP.{SCSeqNum}:IncomeTax.Deduction.SuperannuationContribution.Amount
            
                    ^DDCTNS402 = RP.{SCSeqNum}:OrganisationNameDetails.OrganisationalName.Text
            
                    ^DDCTNS404 = RP.{SCSeqNum}:SuperannuationFundDetails.MemberAccount.Identifier
            
                    ^DDCTNS405 = RP.{SCSeqNum}:Identifiers.AustralianBusinessNumber.Identifier
            
                    ^DDCTNS406 = RP.{SCSeqNum}:Identifiers.TaxFileNumber.Identifier
                    */
                    assertion = (rpscSeqNumContext.DDCTNS407.GetValueOrDefault() > 0 && (string.IsNullOrWhiteSpace(rpscSeqNumContext.DDCTNS402) == true && string.IsNullOrWhiteSpace(rpscSeqNumContext.DDCTNS404) == true || rpscSeqNumContext.DDCTNS405 == null && rpscSeqNumContext.DDCTNS406 == null));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.DDCTNS.000303", Severity = ProcessMessageSeverity.Error,
                            Description = @"Personal superannuation contributions information is required",
                            LongDescription = @"An amount is present at Superannuation deduction claimed for Personal superannuation contributions. Either the Full superannuation fund name and the Superannuation account number are not populated, or the Superannuation fund Australiant Business Number (ABN) and Superannuation fund Tax File Number (TFN) are not populated.",
                            Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.SuperannuationContribution.Amount[@contextRef='" + rpscSeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000303"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS407", Value = GetValueOrEmpty(rpscSeqNumContext.DDCTNS407) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS402", Value = GetValueOrEmpty(rpscSeqNumContext.DDCTNS402) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS404", Value = GetValueOrEmpty(rpscSeqNumContext.DDCTNS404) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS405", Value = GetValueOrEmpty(rpscSeqNumContext.DDCTNS405) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS406", Value = GetValueOrEmpty(rpscSeqNumContext.DDCTNS406) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.DDCTNS.000303

                    #region VR.ATO.DDCTNS.000312

                    /*  VR.ATO.DDCTNS.000312
                    Date of last eligble Superannuation contribution must be within the current financial year.
    
                    Legacy Rule Format:
                    (^DDCTNS403 <> NULL AND (^DDCTNS403 < ^DDCTNS300 OR ^DDCTNS403 > ^DDCTNS301))
        
                    Technical Business Rule Format:
                    (^DDCTNS403 <> NULL AND (^DDCTNS403 < ^DDCTNS300 OR ^DDCTNS403 > ^DDCTNS301))
            
                    Data Elements:
            
                    ^DDCTNS403 = RP.{SCSeqNum}:IncomeTax.Deduction.SuperannuationContributionEligible.Date
            
                    ^DDCTNS300 = RP:Lodgment.PeriodStart.Date
            
                    ^DDCTNS301 = RP:Lodgment.PeriodEnd.Date
                    */
                    assertion = (rpscSeqNumContext.DDCTNS403 != null && (rpscSeqNumContext.DDCTNS403.GetValueOrDefault() < report.DDCTNS300.GetValueOrDefault() || rpscSeqNumContext.DDCTNS403.GetValueOrDefault() > report.DDCTNS301.GetValueOrDefault()));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.DDCTNS.000312",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Date of last eligible superannuation contribution must be within the current financial year",
                            Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.SuperannuationContributionEligible.Date[@contextRef='" + rpscSeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000312" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS403", Value = GetValueOrEmpty(rpscSeqNumContext.DDCTNS403) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS300", Value = GetValueOrEmpty(report.DDCTNS300) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS301", Value = GetValueOrEmpty(report.DDCTNS301) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.DDCTNS.000312

                }
            }

            #endregion // Foreach loop

            #region Repeating RPClthngSeqNumContext
            if (report.RPClthngSeqNumContextCollection != null)
            {
                int itemIndexContext2 = 0;
                foreach (DDCTNS2019.RPClthngSeqNumContext rpClthngSeqNumContext in report.RPClthngSeqNumContextCollection)
                {
                    itemIndexContext2++;

                    #region VR.ATO.DDCTNS.000319

                    /*  VR.ATO.DDCTNS.000319
                    When an amount above zero is provided for Work related clothing expenses amount, the Work related clothing expenses claim type must be provided
    
                    Legacy Rule Format:
                    (^DDCTNS416 > 0 AND ^DDCTNS415 = NULL)
        
                    Technical Business Rule Format:
                    ^DDCTNS416 > 0 AND ^DDCTNS415 = NULL
            
                    Data Elements:
            
                    ^DDCTNS415 = RP.{ClthngSeqNum}:IncomeTax.Deduction.ClothingType.Code
            
                    ^DDCTNS416 = RP.{ClthngSeqNum}:IncomeTax.Deduction.Clothing.Amount
                    */
                    assertion = (rpClthngSeqNumContext.DDCTNS416.GetValueOrDefault() > 0 && rpClthngSeqNumContext.DDCTNS415 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.DDCTNS.000319",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Work related clothing expenses claim type must be provided",
                            LongDescription = @"When an amount above zero is provided for Work related clothing expenses amount, the Work related clothing expenses claim type must be provided",
                            Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.ClothingType.Code[@contextRef='" + rpClthngSeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000319" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS416", Value = GetValueOrEmpty(rpClthngSeqNumContext.DDCTNS416) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "DDCTNS415", Value = GetValueOrEmpty(rpClthngSeqNumContext.DDCTNS415) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.DDCTNS.000319
                }
            }

            #endregion // Foreach loop

            foreach (ProcessMessageDocument currentProcessMessage in response)
            {
                if (currentProcessMessage.Parameters != null)
                {
                    foreach (ProcessMessageParameter currentParameter in currentProcessMessage.Parameters)
                    {
                        if (string.IsNullOrEmpty(currentParameter.Name))
                        {
                            currentParameter.Name = _emptyParameterValue;
                        }

                        if (currentParameter.Name.Length > _maxParameterNameLength)
                        {
                            currentParameter.Name = currentParameter.Name.Substring(0, _maxParameterNameLength - 1);
                        }

                        if (string.IsNullOrEmpty(currentParameter.Value))
                        {
                            currentParameter.Value = _emptyParameterValue;
                        }

                        if (currentParameter.Value.Length > _maxParameterValueLength)
                        {
                            currentParameter.Value = currentParameter.Value.Substring(0, _maxParameterValueLength - 1);
                        }
                    }
                }
            }

            return response;
        }


    }
}
